/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	rPmUpgradeNewMethod( structProperty* property );
int	MGPMrController( structProperty* property, char* path )
{
	char	ignore[]			= "/IGNORE";
	char	upgradeInteractive[]		= "--upgrade-interactive";
	char	resume[]			= "--resume";
	char	ui[]				= "-ui";
	char	backUp[]			= "--back-up";
	char	yes[]				= "--yes";
	char	y[]				= "-y";
	char	f[]				= "-f";
	char	fix[]				= "--fix";
	char	bu[]				= "-bu";
	char	exact[]				= "exact";
	char	id[]				= "MGPMrController";
	char*	command				= NULL;
	char*	configDbValuePtr		= NULL;
	char*	oldPortDir			= NULL;
	char*	oldPortName			= NULL;
	char*	oldPortsDbNewPortDirPtr		= NULL;
	char*	oldPortsDbNewPortNamePtr	= NULL;
	char*	oldPortsDbReasonPtr		= NULL;
	char*	term				= NULL;
	char*	xtermTitle			= NULL;
	float	percentDone			= 0;
	int	bufferSize			= 0xffff;
	int	errorCode			= 0;
	int	oldPortsDbIDX			= 0;
	int	oldPortsDbQTY			= 0;
	int	oldPortsDbTOTALIZER		= 0;

	command		= (char*)calloc( bufferSize,1);
	oldPortDir	= (char*)calloc(512,1);
	oldPortName	= (char*)calloc(512,1);
	xtermTitle	= (char*)calloc( bufferSize,1 );	

	property->cacheDb	= MGdbCreate(
					property->cacheDbFileName,
					property->fieldCacheDbPortDir,
					property->fieldCacheDbAvailablePortName,
					property->fieldCacheDbMakeFileSize,
					property->fieldCacheDbMakeFileTime,
					NULL );
	MGdbDestroy( property->cacheDb );

	property->strikesDb	= MGdbCreate( property->strikesDbFileName, property->fieldStrikesDbPortDir, property->fieldStrikesIDX, NULL );
	MGdbDestroy( property->strikesDb );

	property->commandLineDb	= MGdbOpen( property->commandLineDbFileName );

	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, f, property->fieldCommandLineDbKey, exact ) )
	{
		property->forced	= 1;
		fprintf( stdout, "%s %s info: running in forced update mode\n", id, PACKAGE_VERSION );
		while( fflush( stdout ) );
	}
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, fix, property->fieldCommandLineDbKey, exact ) )
	{
		property->forced	= 1;
		fprintf( stdout, "%s %s info: running in forced update mode\n", id, PACKAGE_VERSION );
		while( fflush( stdout ) );
	}

	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, bu, property->fieldCommandLineDbKey, exact ) )
	{
		property->backUp	= 1;
	} 
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, backUp, property->fieldCommandLineDbKey, exact ) )
	{
		property->backUp	= 1;
	} 
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, y, property->fieldCommandLineDbKey, exact ) )
	{
		property->autoConflicts	= 1;
		property->autoMoved	= 1;
	} 
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, yes, property->fieldCommandLineDbKey, exact ) )
	{
		property->autoConflicts	= 1;
	} 
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, resume, property->fieldCommandLineDbKey, exact ) )
	{
		property->resume	= 1;
	} 
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, ui, property->fieldCommandLineDbKey, exact ) )
	{
		property->interactive	= 1;
	} 
	MGdbGoTop( property->commandLineDb );
	if( MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, upgradeInteractive, property->fieldCommandLineDbKey, exact ) )
	{
		property->interactive	= 1;
	} 

	/*
	 * Need to check configDb for ignored ports and add them to ignoreDb here
	 */
	MGPMrReadConfigure( property );
	property->configDb			= MGdbOpen( property->configDbFileName );
	property->ignoreDb			= MGdbOpen( property->ignoreDbFileName );
	MGdbGoTop( property->configDb );
	while( ( configDbValuePtr = MGdbSeek( property->configDb, property->fieldConfigDbKey, ignore, property->fieldConfigDbValue, exact ) ) )
	{
		MGdbAdd( property->ignoreDb, configDbValuePtr, "set to ignore in pm-020.conf", NULL );
		fprintf( stdout, "%s %s info: Adding %s to ignoreDB reason: %s\n", id, PACKAGE_VERSION, configDbValuePtr, "set to ignore in pm-020.conf" );
		while( fflush( stdout ) );
	}

	MGdbDestroy( property->configDb );
	MGdbDestroy( property->ignoreDb );
	MGdbDestroy( property->commandLineDb );

	while( fflush( stdout ) );
	if( property->log )
	{
		strcpy( command, "echo \"portmanager.log\" " );
		strcat( command, " > /var/log/portmanager.log" );
		system( command );
		strcpy( command, "date >> /var/log/portmanager.log" );
		system( command );
		if( property->buildDependsAreLeaves )
		{
			strcpy( command, "echo \"  running in WITH_BUILD_DEPENDS_ARE_LEAVES mode\" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );

			strcpy( command, "echo \"using this mode is not endorsed by author, if ports\" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );

			strcpy( command, "echo \"fail building due to missing build dependencies\" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );

			strcpy( command, "echo \"in sysutils/portmanager run \"make config\" and shut this mode off\" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );

			strcpy( command, "echo \"then rebuild/reinstall portmanager\" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );

			strcpy( command, "echo \"before contacting author/maintainer\" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );

			strcpy( command, "echo \"=============================================================\" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );
		}
	}
	if( property->buildDependsAreLeaves )
	{
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "%s\n", "running in WITH_BUILD_DEPENDS_ARE_LEAVES mode" );
		fprintf( stdout, "%s\n", "using this mode is not endorsed by author, if ports" );
		fprintf( stdout, "%s\n", "fail building due to missing build dependencies" );
		fprintf( stdout, "%s\n", "in sysutils/portmanager run \"make config\" and shut this mode off" );
		fprintf( stdout, "%s\n", "then rebuild/reinstall portmanager before contacting author/maintainer" );
		fprintf( stdout, "%s\n", DOUBLE_LINES );
	}
	if( path && ( strcmp( path, "SHOWLEAVES" ) == 0 ) )
	{
		if( property->resume == 0 ) /* skip this if portmanager has upgraded/rebuilt its self */
		{
			if( ( errorCode	= MGPMrCreateInstalledDb( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
					"MGPMrCreateInstalledDb", errorCode );
				free( command );
				free( oldPortDir );
				free( oldPortName );
				free( xtermTitle );
				return( 1 );
			}
			if( ( errorCode	= MGPMrCreateAllUpdateStatusDb( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
					"MGPMrCreateAllUpdateStatusDb", errorCode );
				free( command );
				free( oldPortDir );
				free( oldPortName );
				free( xtermTitle );
				return( 1 );
			}
		}

		if( ( errorCode	= MGPMrShowLeaves( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrShowLeaves", errorCode );
			free( command );
			free( oldPortDir );
			free( oldPortName );
			free( xtermTitle );
			return( 1 );
		}
		free( command );
		free( oldPortDir );
		free( oldPortName );
		free( xtermTitle );
		if( property->log )
		{
			strcpy( command, "echo \" end of log \" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );
			strcpy( command, "date >> /var/log/portmanager.log" );
			system( command );
		}
		return( 0 );
	}

	if( path && ( strcmp( path, "SHOWLEAVESDELETE" ) == 0 ) )
	{
		if( property->resume == 0 ) /* skip this if portmanager has upgraded/rebuilt its self */
		{
			property->cacheDb	= MGdbCreate(
							property->cacheDbFileName,
							property->fieldCacheDbPortDir,
							property->fieldCacheDbAvailablePortName,
							property->fieldCacheDbMakeFileSize,
							property->fieldCacheDbMakeFileTime,
							NULL );
			MGdbDestroy( property->cacheDb );
			if( ( errorCode	= MGPMrCreateInstalledDb( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
					"MGPMrCreateInstalledDb", errorCode );
				free( command );
				free( oldPortDir );
				free( oldPortName );
				free( xtermTitle );
				return( 1 );
			}
			if( ( errorCode	= MGPMrCreateAllUpdateStatusDb( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
					"MGPMrCreateAllUpdateStatusDb", errorCode );
				free( command );
				free( oldPortDir );
				free( oldPortName );
				free( xtermTitle );
				return( 1 );
			}
		}

		if( ( property->ignoreDb = MGdbCreate( property->ignoreDbFileName, property->fieldignoreDbPortDir, property->fieldIgnoreDbReason, NULL ) ) == NULL )
		{
			fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->ignoreDbFileName );
			perror( "system" );
			free( command );
			free( oldPortDir );
			free( oldPortName );
			free( xtermTitle );
			if( property->log )
			{
				strcpy( command, "echo \" end of log \" " );
				strcat( command, " >> /var/log/portmanager.log" );
				system( command );
				strcpy( command, "date >> /var/log/portmanager.log" );
				system( command );
			}
			return( 1 );
		}
		MGdbDestroy( property->ignoreDb );
		
		if( ( errorCode	= MGPMrShowLeavesDelete( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrShowLeavesDelete", errorCode );
			free( command );
			free( oldPortDir );
			free( oldPortName );
			free( xtermTitle );
			if( property->log )
			{
				strcpy( command, "echo \" end of log \" " );
				strcat( command, " >> /var/log/portmanager.log" );
				system( command );
				strcpy( command, "date >> /var/log/portmanager.log" );
				system( command );
			}
			return( 1 );
		}
		free( command );
		free( oldPortDir );
		free( oldPortName );
		free( xtermTitle );
		if( property->log )
		{
			strcpy( command, "echo \" end of log \" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );
			strcpy( command, "date >> /var/log/portmanager.log" );
			system( command );
		}
		return( 0 );
	}

	if( path && ( strcmp( path, "PMSTATUS" ) == 0 ) )
	{
		if( ( errorCode	= MGPMrCreateInstalledDb( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrCreateStatusDb", errorCode );
			free( command );
			free( oldPortDir );
			free( oldPortName );
			free( xtermTitle );
			if( property->log )
			{
				strcpy( command, "echo \" end of log \" " );
				strcat( command, " >> /var/log/portmanager.log" );
				system( command );
				strcpy( command, "date >> /var/log/portmanager.log" );
				system( command );
			}
			return( 1 );
		}

		if( ( errorCode = MGPMrMissingDependencies( property ) ) )
		{
			fprintf( stdout, "%s %s error:  %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrMissingDependencies", errorCode );
			free( command );
			free( oldPortDir );
			free( oldPortName );
			free( xtermTitle );
			if( property->log )
			{
				strcpy( command, "echo \" end of log \" " );
				strcat( command, " >> /var/log/portmanager.log" );
				system( command );
				strcpy( command, "date >> /var/log/portmanager.log" );
				system( command );
			}
			return( 1 );
		}

		if( ( errorCode	= MGPMrStatus( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrCreateStatusDb", errorCode );
			free( command );
			free( oldPortDir );
			free( oldPortName );
			free( xtermTitle );
			if( property->log )
			{
				strcpy( command, "echo \" end of log \" " );
				strcat( command, " >> /var/log/portmanager.log" );
				system( command );
				strcpy( command, "date >> /var/log/portmanager.log" );
				system( command );
			}
			return( 1 );
		}
		free( command );
		free( oldPortDir );
		free( oldPortName );
		free( xtermTitle );
		if( property->log )
		{
			strcpy( command, "echo \" end of log \" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );
			strcpy( command, "date >> /var/log/portmanager.log" );
			system( command );
		}
		return(0);
	}

	if( path && strcmp( path, "PMUPGRADE" ) == 0 )
	{
		rPmUpgradeNewMethod( property );
		free( command );
		free( oldPortDir );
		free( oldPortName );
		free( xtermTitle );
		if( property->log )
		{
			strcpy( command, "echo \" end of log \" " );
			strcat( command, " >> /var/log/portmanager.log" );
			system( command );
			strcpy( command, "date >> /var/log/portmanager.log" );
			system( command );
		}
		return( 0 );
	}

	if( path  )
	{
		/*
		 * Single port upgrade
		 */
		if( property->resume == 0 ) /* skip this if --resume  */
		{
			if( ( errorCode	= MGPMrSinglePortCreateStatusDb( property, path ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrSinglePortCreateStatusDb", errorCode );
				free( command );
				free( oldPortDir );
				free( oldPortName );
				if( property->log )
				{
					strcpy( command, "echo \" end of log \" " );
					strcat( command, " >> /var/log/portmanager.log" );
					system( command );
					strcpy( command, "date >> /var/log/portmanager.log" );
					system( command );
				}
				free( xtermTitle );
				return( 1 );
			}
		}
		while( 1 )
		{
			if( ( errorCode	= MGPMrCreateInstalledDb( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrCreateStatusDb", errorCode );
				free( command );
				free( oldPortDir );
				free( oldPortName );
				free( xtermTitle );
				if( property->log )
				{
					strcpy( command, "echo \" end of log \" " );
					strcat( command, " >> /var/log/portmanager.log" );
					system( command );
					strcpy( command, "date >> /var/log/portmanager.log" );
					system( command );
				}
				return( 1 );
			}

			if( ( errorCode	= MGPMrSinglePortStatus( property ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrSinglePortStatus", errorCode );
				free( command );
				free( oldPortDir );
				free( oldPortName );
				free( xtermTitle );
				if( property->log )
				{
					strcpy( command, "echo \" end of log \" " );
					strcat( command, " >> /var/log/portmanager.log" );
					system( command );
					strcpy( command, "date >> /var/log/portmanager.log" );
					system( command );
				}
				return( 1 );
			}
			oldPortDir[0]	= 0;
			oldPortName[0]	= 0;
			if( ( errorCode	= MGPMrBestOldPort( property, oldPortDir, oldPortName ) ) )
			{
				if( errorCode == 100 )
				{
					fprintf( stdout, "%s\n", SINGLE_LINES );
					fprintf( stdout, "%s %s INFO: finished with some ports not updated  if --log was used see /var/log/portmanager.log\n",
						"portmanager", PACKAGE_VERSION );
					while( fflush( stdout ) );
					fprintf( stdout, "%s\n", SINGLE_LINES );
					errorCode = 0;
					break;
				}
				else
				{
					fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrBestOldPort", errorCode );
					free( command );
					free( oldPortDir );
					free( oldPortName );
					free( xtermTitle );
					if( property->log )
					{
						strcpy( command, "echo \" end of log \" " );
						strcat( command, " >> /var/log/portmanager.log" );
						system( command );
						strcpy( command, "date >> /var/log/portmanager.log" );
						system( command );
					}
					return( 1 );
				}
			}
			if( strlen(oldPortDir) > 1 )
			{
				property->oldPortsDb	= MGdbOpen( property->oldPortsDbFileName );
				property->configDb	= MGdbOpen( property->configDbFileName );
				MGdbGoTop( property->configDb );
				configDbValuePtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, oldPortDir,
								property->fieldConfigDbValue, exact );				

				MGdbGoTop( property->oldPortsDb );
				oldPortsDbReasonPtr	= MGdbSeek( property->oldPortsDb, property->fieldOldPortsDbOldPortDir, oldPortDir,
								property->fieldOldPortsDbReason, exact );
				if( oldPortsDbReasonPtr )
				{
					oldPortsDbIDX	= MGdbGetRecno( property->oldPortsDb ) - 1;
					oldPortsDbNewPortNamePtr	= MGdbGet( property->oldPortsDb, oldPortsDbIDX, 
										property->fieldOldPortsDbNewPortName );
					oldPortsDbNewPortDirPtr	= MGdbGet( property->oldPortsDb, oldPortsDbIDX, 
										property->fieldOldPortsDbNewPortDir );
				}
				if( strcmp( oldPortName, "not installed" ) == 0 )
				{
					strcpy( command, "adding " );
					strcat( command, oldPortName );
					strcat( command, " " );
					strcat( command, oldPortDir );
					strcat( command, " " );
				}
				else
				{
					strcpy( command, "updating " );
					strcat( command, oldPortName );
					strcat( command, " " );
					strcat( command, oldPortDir );
					strcat( command, " " );
				}
				if( configDbValuePtr )
				{
					strcat( command, " options " );
					strcat( command, configDbValuePtr );
				}
				if( oldPortsDbReasonPtr )
				{
					strcat( command, " reason: " );
					strcat( command, oldPortsDbReasonPtr );
					strcat( command, " " );
					strcat( command, oldPortsDbNewPortNamePtr );
					strcat( command, " " );
					strcat( command, oldPortsDbNewPortDirPtr );
					strcat( command, " " );
				}
				fprintf( stdout, "%s\n", command );
				fprintf( stdout, "%s\n", SINGLE_LINES );

				/*
				 *  Set xterm title bar indicator
				 */
				oldPortsDbQTY		= MGdbGetRecordQty( property->oldPortsDb );

				if( oldPortsDbQTY - 1 > oldPortsDbTOTALIZER )
				{
					oldPortsDbTOTALIZER	= oldPortsDbQTY - 1;
				}
				percentDone	= 100 - ( 100 * ( ( (float)oldPortsDbQTY - 1 ) / (float)oldPortsDbTOTALIZER ) );
				fprintf( stdout, "percentDone-=>%d = 100 - ( 100 * ( oldPortsDbQTY-=>%d / oldPortsDbTOTALIZER-=>%d ) ) \n",
					(int)percentDone, oldPortsDbQTY-1, oldPortsDbTOTALIZER );
				while( fflush( stdout ) );

				term	= getenv("TERM");
				if( term && strncmp( term, "xterm", 5 ) == 0 && isatty(fileno(stdout)) )
				{
					/*
					pr 81481
					To fix the modification of xterm title, even when redirecting, one can
					test isatty(fileno(stdout)) before printing the escape sequence.
					*/
					strcpy( xtermTitle, "portmanager" );
					strcat( xtermTitle, " done:% " );
					strcat( xtermTitle, MGrIntToString( (long)percentDone ) );
					strcat( xtermTitle, " updating " );
					strcat( xtermTitle, oldPortName );
					strcat( xtermTitle, " " );
					strcat( xtermTitle, oldPortDir );
					printf( "%c]0;%s%c", '\033', xtermTitle, '\007' );
				}
				if( property->log )
				{
					strcpy( command, "echo \" " );
					if( strcmp( oldPortName, "not installed" ) == 0 )
					{
						strcat( command, "adding " );
						strcat( command, oldPortName );
						strcat( command, " " );
						strcat( command, oldPortDir );
						strcat( command, " " );
					}
					else
					{
						strcat( command, "updating " );
						strcat( command, oldPortName );
						strcat( command, " " );
						strcat( command, oldPortDir );
						strcat( command, " " );
					}
					if( configDbValuePtr )
					{
						strcat( command, " options " );
						strcat( command, configDbValuePtr );
					}
					if( oldPortsDbReasonPtr )
					{
						strcat( command, " " );
						strcat( command, oldPortsDbReasonPtr );
						strcat( command, " " );
						strcat( command, oldPortsDbNewPortNamePtr );
						strcat( command, " " );
						strcat( command, oldPortsDbNewPortDirPtr );
						strcat( command, " " );
					}
					strcat( command, "\" >> /var/log/portmanager.log" );
					system( command );
				}
				MGdbDestroy( property->configDb );
				MGdbDestroy( property->oldPortsDb );
			}
			else
			{
				/*
				 *  Set xterm title bar indicator
				 *
				 * pr 81481
				 * To fix the modification of xterm title, even when redirecting, one can
				 * test isatty(fileno(stdout)) before printing the escape sequence.
				 */
				if( term && strncmp( term, "xterm", 5 ) == 0 && isatty(fileno(stdout)) )
				{
					strcpy( xtermTitle, "portmanager" );
					strcat( xtermTitle, " " );
					strcat( xtermTitle, " finished" );
					printf( "%c]0;%s%c", '\033', xtermTitle, '\007' );
				}
				fprintf( stdout, "%s %s INFO: all ports are up to date\n", "portmanager", PACKAGE_VERSION );
				while( fflush( stdout ) );
				break;
			}
			MGPMrUpdate( property, oldPortDir, oldPortName );
		}
	}
	if( property->log )
	{
		strcpy( command, "echo \" end of log \" " );
		strcat( command, " >> /var/log/portmanager.log" );
		system( command );
		strcpy( command, "date >> /var/log/portmanager.log" );
		system( command );
	}
	free( command );
	free( oldPortDir );
	free( oldPortName );
	free( xtermTitle );
	return( errorCode );
}

/*******************************************************/
int	rPmUpgradeNewMethod( structProperty* property )
/*******************************************************/
{
	char	exact[]				= "exact";
	char	id[]				= "rPmUpgradeNewMethod";
	char*	command				= NULL;
	char*	configDbValuePtr		= NULL;
	char*	oldPortDir			= NULL;
	char*	oldPortName			= NULL;
	char*	oldPortsDbNewPortDirPtr		= NULL;
	char*	oldPortsDbNewPortNamePtr	= NULL;
	char*	oldPortsDbReasonPtr		= NULL;
	char*	term				= NULL;
	char*	xtermTitle			= NULL;
	float	percentDone			= 0;
	int	bufferSize			= 0xffff;
	int	errorCode			= 0;
	int	oldPortsDbIDX			= 0;
	int	oldPortsDbQTY			= 0;
	int	oldPortsDbTOTALIZER		= 0;

	command		= (char*)calloc( bufferSize,1);
	oldPortDir	= (char*)calloc(255,1);
	oldPortName	= (char*)calloc(255,1);
	xtermTitle	= (char*)calloc( bufferSize,1 );	

	property->pmMode	= 1; /* fake like we are in single port update */
	if( property->resume == 0 ) /* skip this if portmanager has upgraded/rebuilt its self */
	{
		property->cacheDb	= MGdbCreate(
						property->cacheDbFileName,
						property->fieldCacheDbPortDir,
						property->fieldCacheDbAvailablePortName,
						property->fieldCacheDbMakeFileSize,
						property->fieldCacheDbMakeFileTime,
						NULL );
		MGdbDestroy( property->cacheDb );
		if( ( errorCode	= MGPMrCreateInstalledDb( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
				"MGPMrCreateInstalledDb", errorCode );
			free( command );
			free( oldPortDir );
			free( xtermTitle );
			return( 1 );
		}
		if( ( errorCode	= MGPMrCreateAllUpdateStatusDb( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION,
				"MGPMrCreateAllUpdateStatusDb", errorCode );
			return( 1 );
		}
	}
	while( 1 )
	{
		if( ( errorCode	= MGPMrCreateInstalledDb( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrCreateStatusDb", errorCode );
			free( command );
			free( oldPortDir );
			free( xtermTitle );
			return( 1 );
		}

		if( ( errorCode	= MGPMrSinglePortStatus( property ) ) )
		{
			fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrSinglePortStatus", errorCode );
			return( 1 );
		}
		oldPortDir[0]	= 0;
		oldPortName[0]	= 0;
		if( ( errorCode	= MGPMrBestOldPort( property, oldPortDir, oldPortName ) ) )
		{
			if( errorCode == 100 )
			{
				fprintf( stdout, "%s\n", SINGLE_LINES );
				fprintf( stdout, "%s %s INFO: finished with some ports not updated  if --log was used see /var/log/portmanager.log\n",
					"portmanager", PACKAGE_VERSION );
				while( fflush( stdout ) );
				fprintf( stdout, "%s\n", SINGLE_LINES );
				errorCode = 0;
				break;
			}
			else
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGPMrBestOldPort", errorCode );
				return( 1 );
			}
		}
		if( strlen(oldPortDir) > 1 )
		{
			property->oldPortsDb	= MGdbOpen( property->oldPortsDbFileName );
			property->configDb	= MGdbOpen( property->configDbFileName );
			MGdbGoTop( property->configDb );
			configDbValuePtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, oldPortDir,
							property->fieldConfigDbValue, exact );				

			MGdbGoTop( property->oldPortsDb );
			oldPortsDbReasonPtr	= MGdbSeek( property->oldPortsDb, property->fieldOldPortsDbOldPortDir, oldPortDir,
							property->fieldOldPortsDbReason, exact );
			if( oldPortsDbReasonPtr )
			{
				oldPortsDbIDX	= MGdbGetRecno( property->oldPortsDb ) - 1;
				oldPortsDbNewPortNamePtr	= MGdbGet( property->oldPortsDb, oldPortsDbIDX, 
									property->fieldOldPortsDbNewPortName );
				oldPortsDbNewPortDirPtr	= MGdbGet( property->oldPortsDb, oldPortsDbIDX, 
									property->fieldOldPortsDbNewPortDir );
			}
			if( strcmp( oldPortName, "not installed" ) == 0 )
			{
				strcpy( command, "adding " );
				strcat( command, oldPortName );
				strcat( command, " " );
				strcat( command, oldPortDir );
				strcat( command, " " );
			}
			else
			{
				strcpy( command, "updating " );
				strcat( command, oldPortName );
				strcat( command, " " );
				strcat( command, oldPortDir );
				strcat( command, " " );
			}
			if( configDbValuePtr )
			{
				strcat( command, " options " );
				strcat( command, configDbValuePtr );
			}
			if( oldPortsDbReasonPtr )
			{
				strcat( command, " reason: " );
				strcat( command, oldPortsDbReasonPtr );
				strcat( command, " " );
				strcat( command, oldPortsDbNewPortNamePtr );
				strcat( command, " " );
				strcat( command, oldPortsDbNewPortDirPtr );
				strcat( command, " " );
			}
			fprintf( stdout, "%s\n", command );
			fprintf( stdout, "%s\n", SINGLE_LINES );

			/*
			 *  Set xterm title bar indicator
			 */
			oldPortsDbQTY		= MGdbGetRecordQty( property->oldPortsDb );

			if( oldPortsDbQTY - 1 > oldPortsDbTOTALIZER )
			{
				oldPortsDbTOTALIZER	= oldPortsDbQTY - 1;
			}
			percentDone	= 100 - ( 100 * ( ( (float)oldPortsDbQTY - 1 ) / (float)oldPortsDbTOTALIZER ) );
			fprintf( stdout, "percentDone-=>%d = 100 - ( 100 * ( oldPortsDbQTY-=>%d / oldPortsDbTOTALIZER-=>%d ) ) \n",
				(int)percentDone, oldPortsDbQTY-1, oldPortsDbTOTALIZER );
			while( fflush( stdout ) );

			term	= getenv("TERM");
			if( term && strncmp( term, "xterm", 5 ) == 0 && isatty(fileno(stdout)) )
			{
				/*
				pr 81481
				To fix the modification of xterm title, even when redirecting, one can
				test isatty(fileno(stdout)) before printing the escape sequence.
				*/
				strcpy( xtermTitle, "portmanager" );
				strcat( xtermTitle, " done:% " );
				strcat( xtermTitle, MGrIntToString( (long)percentDone ) );
				strcat( xtermTitle, " updating " );
				strcat( xtermTitle, oldPortName );
				strcat( xtermTitle, " " );
				strcat( xtermTitle, oldPortDir );
				printf( "%c]0;%s%c", '\033', xtermTitle, '\007' );
			}
			if( property->log )
			{
				strcpy( command, "echo \" " );
				if( strcmp( oldPortName, "not installed" ) == 0 )
				{
					strcat( command, "adding " );
					strcat( command, oldPortName );
					strcat( command, " " );
					strcat( command, oldPortDir );
					strcat( command, " " );
				}
				else
				{
					strcat( command, "updating " );
					strcat( command, oldPortName );
					strcat( command, " " );
					strcat( command, oldPortDir );
					strcat( command, " " );
				}
				if( configDbValuePtr )
				{
					strcat( command, " options " );
					strcat( command, configDbValuePtr );
				}
				if( oldPortsDbReasonPtr )
				{
					strcat( command, " " );
					strcat( command, oldPortsDbReasonPtr );
					strcat( command, " " );
					strcat( command, oldPortsDbNewPortNamePtr );
					strcat( command, " " );
					strcat( command, oldPortsDbNewPortDirPtr );
					strcat( command, " " );
				}
				strcat( command, "\" >> /var/log/portmanager.log" );
				system( command );
			}
			MGdbDestroy( property->configDb );
			MGdbDestroy( property->oldPortsDb );
		}
		else
		{
			/*
			 *  Set xterm title bar indicator
			 *
			 * pr 81481
			 * To fix the modification of xterm title, even when redirecting, one can
			 * test isatty(fileno(stdout)) before printing the escape sequence.
			 */
			if( term && strncmp( term, "xterm", 5 ) == 0 && isatty(fileno(stdout)) )
			{
				strcpy( xtermTitle, "portmanager" );
				strcat( xtermTitle, " " );
				strcat( xtermTitle, " finished" );
				printf( "%c]0;%s%c", '\033', xtermTitle, '\007' );
			}
			fprintf( stdout, "%s %s INFO: all ports are up to date\n", "portmanager", PACKAGE_VERSION );
			while( fflush( stdout ) );
			break;
		}
		MGPMrUpdate( property, oldPortDir, oldPortName );
	}
	if( property->log )
	{
		strcpy( command, "echo \" end of log \" " );
		strcat( command, " >> /var/log/portmanager.log" );
		system( command );
		strcpy( command, "date >> /var/log/portmanager.log" );
		system( command );
	}
	free( oldPortName );
	free( command );
	free( oldPortDir );
	free( xtermTitle );
	return( 0 );
}
