/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	rrDependencyParse( structProperty* property, char* buffer, char* availablePortName, char* availablePortDir, char* dependType, int* counterPtr );
int	rrFillDependencies( structProperty* property, char* availablePortName, char* availablePortDir, int* counterPtr );
int	rrFindDepends( structProperty* property, char* availablePortDir, char* availablePortName, int* counterPtr );
int	rrGetPortName( structProperty* property, char* portDir, char* portName );
int	rrParse(  structProperty* property, char* availablePortName, char* availablePortDir, char* dependType, char* buffer, int* counterPtr );

int	MGPMrCreateAllUpdateStatusDb( structProperty* property )
{
	FILE*	dependencyDbStream		= NULL;
	char	exact[]				= "exact";
	char	id[]				= "MGPMrCreateAllUpdateStatusDb";
	char*	availablePortDir		= NULL;
	char*	availablePortName		= NULL;
	char*	installedPortsDbPortDirPtr	= NULL;
	int	bufferSize			= 0xffff;
	int	counter				= 0;
	int	errorCode			= 0;
	int	installedPortsDbIDX		= 0;
	int	installedPortsDbQTY		= 0;
	int*	counterPtr			= NULL;
	
	counterPtr	= &counter;

	availablePortDir	= (char*)calloc( bufferSize, 1 );
	availablePortName	= (char*)calloc( bufferSize, 1 );

	fprintf( stdout,"%s\n", SINGLE_LINES );
	if( property->forced )
	{
		fprintf( stdout,"%s %s: Collecting installed port data \"forced mode\"\n", "portmanager", PACKAGE_VERSION );
	}
	else
	{
		fprintf( stdout,"%s %s: Collecting installed port data \n", "portmanager", PACKAGE_VERSION );

	}
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	property->availablePortsDb	= MGdbCreate(
						property->availablePortsDbFileName,
						property->fieldAvailablePortsDbPortName,
						property->fieldAvailablePortsDbPortDir,
						property->fieldAvailablePortsDbFlag,
						NULL);

	/*
	 * creates dependencyDb with header record, same as MGdbCreate.  In this case records will be added by
	 * fprint'ing to dependencyDbStream instead of using MGdbAdd because speed is important here so we don't
	 * want a dependencyDb object yet.
	 */
	dependencyDbStream	= fopen( property->availableDependenciesDbFileName, "w" );
	if( dependencyDbStream == 0 )
	{
		fprintf( stdout, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, property->availableDependenciesDbFileName );
		perror( "system message" );
		return( 1 );
	}
	fprintf( dependencyDbStream, "%s%c%s%c%s%c%s%c%s%c%c", "fieldAvailableDependenciesDbPortDir", 0, "fieldAvailableDependenciesDbPortName", 0,
		"fieldAvailableDependenciesDbDependencyDir", 0,	"fieldAvailableDependenciesDbDependencyName", 0,
		"fieldAvailableDependenciesDbDependType", 0, LINEFEED );
	fclose( dependencyDbStream );

	property->availableDependenciesDb	= MGdbOpen( property->availableDependenciesDbFileName );
	property->configDb			= MGdbOpen( property->configDbFileName );
	property->installedPortsDb		= MGdbOpen( property->installedPortsDbFileName );

	installedPortsDbQTY		= MGdbGetRecordQty( property->installedPortsDb );
	counter				= installedPortsDbQTY;
	installedPortsDbIDX		= 1;
	while( installedPortsDbIDX < installedPortsDbQTY )
	{
		installedPortsDbPortDirPtr	= MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortDir );
		MGdbGoTop( property->availablePortsDb );
		if( MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, installedPortsDbPortDirPtr,
			property->fieldAvailablePortsDbPortDir, exact ) )
		{
			/*
			 * this port's dependency info allready gathered
			 */
			 installedPortsDbIDX++;
			 continue;
			 
		}
		strcpy( availablePortDir, installedPortsDbPortDirPtr );
		if( ( errorCode = rrFindDepends( property, availablePortDir, availablePortName, counterPtr ) ) == 100 )
		{
			MGdbDelete( property->installedPortsDb, installedPortsDbIDX );
			installedPortsDbQTY--;
		}
		installedPortsDbIDX++;
	}
	MGdbDestroy( property->availablePortsDb );
	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->availableDependenciesDb );
	MGdbDestroy( property->configDb );
	free( availablePortDir );
	free( availablePortName );
	return( 0 );
}

int	rrFindDepends( structProperty* property, char* availablePortDir, char* availablePortName, int* counterPtr )
{
	char	id[]			= "rrFindDepends";
	int	availablePortsDbIDX	= 0;
	int	errorCode		= 0;
	int	availablePortsDbQTY	= 0;

	if( ( errorCode = rrGetPortName( property, availablePortDir, availablePortName ) ) )
	{
/*
		fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rrGetPortName", errorCode );
*/
		return( 100 );
	}

	MGdbAdd( property->availablePortsDb, availablePortName, availablePortDir, "0", NULL);
	if( ( errorCode = rrFillDependencies( property, availablePortName, availablePortDir, counterPtr ) ) )
	{
		fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rrFillDependencies", errorCode );
		return( 1 );
	}

	/*
 	 * fill dependencies for each record in property->availablePortsDb
 	 */		
	availablePortsDbQTY	= MGdbGetRecordQty( property->availablePortsDb );
	availablePortsDbIDX	= 1;
	while( availablePortsDbIDX < availablePortsDbQTY )
	{
		if( MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbFlag )
			&&
			strncmp( MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbFlag ), "0", 1 ) == 0 )
		{
			strcpy( availablePortDir, MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbPortDir ) );
			strcpy( availablePortName, MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbPortName ) );
			rrFillDependencies( property, availablePortName, availablePortDir, counterPtr );
			availablePortsDbQTY	= MGdbGetRecordQty( property->availablePortsDb );
			availablePortsDbIDX	= 0;
		}
		availablePortsDbIDX++;
	}
	return( 0 );
}

int	rrGetPortName( structProperty* property, char* portDir, char* portName )
{
	DIR*	dirStream	= NULL;
	FILE*	makeStream			= NULL;
	char	exact[]				= "exact";
	char	id[]				= "rrGetPortName";
	char*	availablePortsDbPortNamePtr	= NULL;
	char*	charPtr				= NULL;
	char*	command				= NULL;
	char*	optionsPtr			= NULL;
	int	answer				= 0;
	int	bufferSize			= 0xffff;

	command		= (char*)calloc( bufferSize, 1 );

	MGdbGoTop( property->availablePortsDb );
	availablePortsDbPortNamePtr	= MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, portDir,
						property->fieldAvailablePortsDbPortName, exact );

	if( availablePortsDbPortNamePtr == NULL )
	{
		/*
		 * see if this port has anything special in configDb
		 */
		MGdbGoTop( property->configDb );
		optionsPtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, portDir, property->fieldConfigDbValue, exact );
		strcpy( command, PORTSDIR );
		strcat( command, portDir );
		dirStream	= opendir( command );
		if( dirStream == 0 )	/* bad dir */
		{
			MGdbGoTop( property->installedPortsDb );
			strcpy( portName, MGdbSeek( property->installedPortsDb,
						property->fieldInstalledPortsDbPortDir,
						portDir,
						property->fieldInstalledPortsDbPortName,
						exact ) );
			fprintf( stdout,"%s\n", SINGLE_LINES );
			fprintf( stdout,
				"%s %s is no longer in the ports collection, see /usr/ports/MOVED\nthis port must be removed for portmanager to continue\nis it OK to remove %s from your system? (yes/no/auto yes to all) [y/n/a] [y]\n",
				portName, portDir, portName );
			while( fflush( stdout ) );

			if( property->autoMoved == 0 )
			{
				answer	= getc(stdin);

				switch(answer)
				{
					case KEY_ENTER:
					{
						break;
					}
					case KEY_A:
					{
						answer	= getc(stdin);
						property->autoMoved = 1;
						break;
					}
					case KEY_a:
					{
						property->autoMoved = 1;
						answer	= getc(stdin);
						break;
					}
					case KEY_Y:
					{
						answer	= getc(stdin);
						break;
					}
					case KEY_y:
					{
						answer	= getc(stdin);
						break;
					}
					case KEY_N:
					{
						answer	= getc(stdin);
						fprintf( stdout, "you will have to deal with this manually then, portmanager shutting down\n" );
						while( fflush( stdout ) );
						exit(0);
					}
					case KEY_n:
					{
						answer	= getc(stdin);
						fprintf( stdout, "you will have to deal with this manually then, portmanager shutting down\n" );
						while( fflush( stdout ) );
						exit(0);
					}
					default:
					{
						fprintf( stdout, "Invalid option. Please enter either Y to upgrade or N to skip\n");
						while( fflush( stdout ) );
						answer	= getc(stdin);
					}
				}
			}
			fprintf( stdout, "removing: %s\n", portName );
			strcpy( command, "pkg_delete -f " );
			strcat( command, portName );
			fprintf( stdout,"%s\n", command );
			fprintf( stdout,"%s\n", SINGLE_LINES );
			while( fflush( stdout ) );
			system( command );
			free( command );
			return( 100 );
		}
		else
		{
			closedir( dirStream );
		}
		strcpy( command, "cd " );
		strcat( command, PORTSDIR );
		strcat( command, portDir );
		strcat( command, "; make " );
		strcat( command, " -V PKGNAME " );
		if( optionsPtr )
		{
			strcat( command, optionsPtr );
		}
		makeStream	= popen( command, "r" );
		if( makeStream == 0 )
		{
			fprintf( stdout, "%s error: reading make -V PKGNAME into availablePortName\n", id );
			perror( "system message" );
			return( 1 );
		}
		portName[0]	= 0;
		fread( portName, bufferSize-1, 1, makeStream );
		if( portName == NULL )
		{
			fprintf( stdout, "%s error: reading make -V PKGNAME into availablePortName\n", id );
			perror( "system message" );
			return( 1 );
		}
		charPtr	= portName;
		while( charPtr < portName + (bufferSize-1) )
		{
			if( charPtr[0]	== SPACE || charPtr[0] == TAB || charPtr[0] == LINEFEED || charPtr[0] == ':' )
			{
				charPtr[0]	= 0;
				break;
			}
			charPtr++;
		}
		pclose( makeStream );
	}
	else
	{
		MGdbGoTop( property->availablePortsDb );
		strcpy( portName,  MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, portDir,
					property->fieldAvailablePortsDbPortName, exact ) );
	}
	free( command );
	return( 0 );
}

int	rrFillDependencies( structProperty* property, char* availablePortName, char* availablePortDir, int* counterPtr )
{
	char	BUILD_DEPENDS[]		= "BUILD_DEPENDS";
	char	RUN_DEPENDS[]		= "RUN_DEPENDS";
	char	exact[]			= "exact";
	char	id[]			= "rrFillDependencies";
	char*	availablePortDirPtr	= NULL;
	char*	buffer			= NULL;
	int	availablePortsDbIDX	= 0;
	int	bufferSize		= 0xffff;

	buffer		= calloc( bufferSize + 1, 1 );

	rrDependencyParse( property, buffer, availablePortName, availablePortDir, BUILD_DEPENDS, counterPtr );

	buffer[0]	= 0;

	rrDependencyParse( property, buffer, availablePortName, availablePortDir, RUN_DEPENDS, counterPtr );

	free( buffer );

	/*
	 * Flag availablePortsDb.availablePortName as "1" after dependencies added
	 */
	if( strlen(availablePortDir) < 3 )
	{
		fprintf( stdout, "%s error: strlen(availablePortDir)-=>%d\n", id, strlen(availablePortDir) );
		return( 1 );
	}
	MGdbGoTop( property->availablePortsDb );
	availablePortDirPtr		= MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, availablePortDir,
						property->fieldAvailablePortsDbPortDir, exact );
	availablePortsDbIDX		= MGdbGetRecno( property->availablePortsDb ) - 1;
	property->availablePortsDb	= MGdbDelete( property->availablePortsDb, availablePortsDbIDX );
	MGdbAdd( property->availablePortsDb, availablePortName, availablePortDir, "1", NULL);

	return( 0 );
}

int	rrDependencyParse( structProperty* property, char* buffer, char* availablePortName, char* availablePortDir, char* dependType, int* counterPtr )
{
	FILE*	makeStream	= NULL;
	char	exact[]		= "exact";
	char	id[]		= "rrDependencyParse";
	char*	command		= NULL;
	char*	optionsPtr	= NULL;
	int	bufferSize	= 0xffff;

	/*
	 * port make config option WITH_BUILD_DEPENDS_ARE_LEAVES set.
	 * so not traking build dependencies
	 */
	if( property->buildDependsAreLeaves && strcmp( dependType, "BUILD_DEPENDS" ) == 0 )
	{
		return( 0 );
	}

	command	= (char*)calloc( bufferSize, 1 );
	strcpy( command, "cd " );
	strcat( command, PORTSDIR );
	strcat( command, availablePortDir );
	strcat( command, "; make -V " );

	if( strcmp( dependType, "BUILD_DEPENDS" ) == 0 )
	{
		strcat( command, "BUILD_DEPENDS -V EXTRACT_DEPENDS -V PATCH_DEPENDS -V FETCH_DEPENDS " );
	}
	else
	{
		strcat( command, "RUN_DEPENDS -V LIB_DEPENDS -V DEPENDS " );
	}
	/*
	 * see if this port has anything special in configDb
	 */
	MGdbGoTop( property->configDb );
	optionsPtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, availablePortDir, property->fieldConfigDbValue, exact );
	if( optionsPtr )
	{
		strcat( command, optionsPtr );
	}
	makeStream	= popen( command, "r" );
	if( makeStream == 0 )
	{
		fprintf( stdout, "%s error: reading %s into buffer\n", id, command );
		perror( "system message" );
		return( 1 );
	}	
	fread( buffer, bufferSize-1, 1, makeStream );
	if( strlen( buffer ) > 1 )
	{
		rrParse( property, availablePortName, availablePortDir, dependType, buffer, counterPtr );
	}
	pclose( makeStream );
	free( command );
	return( 0 );
}

int	rrParse(  structProperty* property, char* availablePortName, char* availablePortDir, char* dependType, char* buffer, int* counterPtr )
{
	FILE*	dependencyDbStream	= NULL;
	char	exact[]			= "exact";
	char	id[]			= "rrParse";
	char*	dependencyPortDir	= NULL;
	char*	dependencyPortName	= NULL;
	char*	optionsPtr		= NULL;
	int	bufferSize		= 0xffff;
	int	idx1			= 0;
	int	idx2			= 0;

	dependencyPortDir	= (char*)calloc( bufferSize, 1 );
	dependencyPortName	= (char*)calloc( bufferSize, 1 );
	
	bufferSize	= strlen( buffer );
	idx1		= 0;
	idx2		= 0;

	/*
	 * displaying run dependencies ( every other record ) here....
	 */
	if( strcmp( dependType, "RUN_DEPENDS" ) == 0 )
	{
		counterPtr[0]--;

		MGdbGoTop( property->configDb );
		optionsPtr	= MGdbSeek( property->configDb, property->fieldConfigDbKey, availablePortDir, property->fieldConfigDbValue, exact );
		if( optionsPtr )
		{
			fprintf( stdout, "%05d %s %s %s\n", counterPtr[0], availablePortName, availablePortDir, optionsPtr );
			while( fflush( stdout ) );
		}
		else
		{
			fprintf( stdout, "%05d %s %s\n", counterPtr[0], availablePortName, availablePortDir );
		}
		optionsPtr	= NULL;
	}
	dependencyDbStream	= fopen( property->availableDependenciesDbFileName, "a" );
	if( dependencyDbStream == 0 )
	{
		fprintf( stdout, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, property->availableDependenciesDbFileName );
		perror( "system message" );
		return( 1 );
	}
	while( idx1 < bufferSize )
	{
		if( buffer[idx1] == ':' )
		{
			idx1++;
			idx2	= idx1;
			while( idx2 < bufferSize && buffer[idx2] != ' ' && buffer[idx2] != LINEFEED && buffer[idx2] != TAB && buffer[idx2] != ':' )
			{
				idx2++;
			}
			buffer[idx2]		= 0;
			if( idx1 + 11 < bufferSize )
			{
				strcpy( dependencyPortDir, buffer + idx1 + 10 );
			}
			else
			{
				assert(0);
			}
			rrGetPortName( property, dependencyPortDir, dependencyPortName );
			/*
			 * add dependency to availableDependenciesDb may contain dupes
			 */
			fprintf( dependencyDbStream, "%s%c%s%c%s%c%s%c%s%c%c", availablePortDir, 0, availablePortName, 0, dependencyPortDir,
				0, dependencyPortName, 0, dependType, 0, LINEFEED );
			MGdbGoTop( property->configDb );
			optionsPtr = MGdbSeek( property->configDb, property->fieldConfigDbKey, dependencyPortDir, property->fieldConfigDbValue, exact );
/*
			if( optionsPtr )
			{
				fprintf( stdout, "\t%s %s %s\n", dependencyPortName, dependencyPortDir, optionsPtr );
			}
			else
			{
				fprintf( stdout, "\t%s %s\n", dependencyPortName, dependencyPortDir );
			}
			while( fflush( stdout ) );
*/
			/*
			 * if dependency not in availablePortsDb add it
			 */
			MGdbGoTop( property->availablePortsDb );
			if( MGdbSeek( property->availablePortsDb, property->fieldAvailablePortsDbPortDir, dependencyPortDir,
				property->fieldAvailablePortsDbPortDir, exact ) == 0 )
			{
				MGdbAdd( property->availablePortsDb, dependencyPortName, dependencyPortDir, "0", NULL);
			}
		}
		idx1++;
	}	
	fclose( dependencyDbStream );
	free( dependencyPortDir );
	free( dependencyPortName );
	return(0);
}
