/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	rAddDependencies( structProperty* property, char* installedPortDir, char* installedPortNamePtr );
int	rGetPortDir(char* installedPortNamePtr, char* installedPortDir );
int	rVerifyContentsFile( char* installedPortNamePtr );

int	MGPMrCreateInstalledDb( structProperty* property )
{
	FILE*		dependencyDbStream	= NULL;
	char		id[]			= "MGPMrCreateInstalledDb";
	char		installedPortDir[256];
	char		pkgDir[]		= PKGDBDIR;
	char*		installedPortNamePtr	= NULL;
	classDir*	dir			= NULL;
	int		dirMode			= 7;
	int		errorCode		= 0;
	
	property->installedPortsDb	= MGdbCreate(
						property->installedPortsDbFileName,
						property->fieldInstalledPortsDbPortName,
						property->fieldInstalledPortsDbPortDir,
						property->fieldInstalledPortsDbFlag,
						NULL );

	property->oldPortsDb		= MGdbCreate(
						property->oldPortsDbFileName,
						property->fieldOldPortsDbOldPortDir,
						property->fieldOldPortsDbOldPortName,
						property->fieldOldPortsDbReason,
						property->fieldOldPortsDbNewPortName,
						property->fieldOldPortsDbNewPortDir,
						NULL );

	if( ( dependencyDbStream = fopen( property->dependencyPortsDbFileName, "w" ) ) == 0 )
	{
		fprintf( stderr, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, property->dependencyPortsDbFileName );
		perror( "system message" );
		assert(0);
	}
	fprintf( dependencyDbStream, 
		"%s%c%s%c%s%c%s%c%c", "fieldDependencyDbPortDir", 0, "fieldDependencyDbPortName", 0, "fieldDependencyDbDependencyPortDir", 0,
		"fieldDependencyDbDependencyPortName", 0, LINEFEED );
	fclose( dependencyDbStream );

	dir				= MGdirCreate( dirMode, pkgDir );
	installedPortNamePtr		= MGdirGet(dir);
	if( ( errorCode = rVerifyContentsFile( installedPortNamePtr ) ) )
	{
		fprintf(stderr, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rVerifyContentsFile", errorCode);
		return( 1 );
	}

	while( installedPortNamePtr )
	{
		if( ( errorCode = rGetPortDir( installedPortNamePtr, installedPortDir ) ) )
		{
			fprintf(stderr, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "rGetPortDir", errorCode);
			return( 1 );
		}
		MGdbAdd( property->installedPortsDb, installedPortNamePtr, installedPortDir, "0", NULL);

		rAddDependencies( property, installedPortDir, installedPortNamePtr );

		MGdirSkip(dir);
		if( ( installedPortNamePtr = MGdirGet(dir) ) == 0 )
		{
			break;
		}

		if( ( errorCode	= rVerifyContentsFile( installedPortNamePtr ) ) )
		{
			fprintf(stderr, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "PMGRrVerifyContentsFile", errorCode);
			return( 1 );
		}
	}

	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->oldPortsDb );
	MGdirDestroy( dir );
	return( 0 );
}

int	rAddDependencies( structProperty* property, char* installedPortDir, char* installedPortNamePtr )
{
	FILE*	contentsFileStream;
	FILE*	dependencyDbStream;
	char	id[]			= "PMGRrAddDependencies";
	char*	buffPtr			= 0;
	char*	buffer			= NULL;
	char*	contentsFileName	= NULL;
	char*	portDependencyName	= NULL;
	char*	portDependencyDir	= NULL;
	int	bufferSize		= 0;
	int	stringSize		= 0;
	contentsFileName	= (char*)malloc( 255 );
	strcpy(contentsFileName, PKGDBDIR);
	strcat(contentsFileName, "/");
	strcat(contentsFileName, installedPortNamePtr );
	strcat(contentsFileName, "/");
	strcat(contentsFileName, "+CONTENTS");

	/****************************************/
	/* get +CONTENTS file into buffer	*/
	/****************************************/
	if( ( contentsFileStream = fopen( contentsFileName, "r" ) ) == 0 )
	{
		fprintf( stderr, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, contentsFileName );
		perror( "system message" );
		assert(0);
	}
	bufferSize	= MGrFileSize( contentsFileName );
	buffer		= (char*)calloc( bufferSize + 1, 1 );
	buffer[bufferSize-1]	= 0;
	fread(buffer, 1, bufferSize, contentsFileStream );
	buffPtr	= buffer;
	portDependencyName	= buffer;
	if( ( dependencyDbStream = fopen( property->dependencyPortsDbFileName, "a" ) ) == 0 )
	{
		fprintf( stderr, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, property->dependencyPortsDbFileName );
		perror( "system message" );
		assert(0);
	}

	while( ( portDependencyName = strnstr( buffPtr, "@pkgdep ", bufferSize - (buffPtr - buffer) ) ) )
	{
		portDependencyName			+= strlen( "@pkgdep " ); 
		stringSize			= 0;
		while( portDependencyName[stringSize] != LINEFEED && stringSize < 256 )
		{
			stringSize++;
		}		

		portDependencyName[stringSize]	= 0;

		buffPtr 			= portDependencyName + stringSize + 1;

		portDependencyDir		= strstr( buffPtr, "@comment DEPORIGIN:" );
		portDependencyDir		+= strlen( "@comment DEPORIGIN:" );
		stringSize			= strstr( portDependencyDir, "\n" ) - portDependencyDir;
		portDependencyDir[stringSize]	= 0;

		if( strncmp( buffPtr, "@comment DEPORIGIN:",  strlen( "@comment DEPORIGIN:" ) ) )
		{
			fprintf( stdout, 
				"%s %s error: %s/%s \"@comment DEPORIGIN:\" is missing from +CONTENTS file, recommend you manually remove this port:\n",
				id, PACKAGE_VERSION, PKGDBDIR, installedPortNamePtr );
			fprintf( stdout, "pkg_delete -f %s/%s then find it in /usr/ports/{category}/{portname} and reinstall\n",
				PKGDBDIR, installedPortNamePtr );
			return( 1 );
		}

		buffPtr 			= portDependencyDir + stringSize + 1;

		portDependencyDir--;
		portDependencyDir[0]		= '/';

		/*
		*
		* Just a note: Turns out that I had to run while( fflush( dbFileStream ) ) in MGdbAdd to prevent
		* random LINEFEEDS from being placed into the data base, that may pop up again somewhere so I'll
		* leave this comment here in the hope it saves some aggravation in the future
		*
		*/ 

		/*
		* 0.3.0 Code to speed up initial db creation
		*
		* 1. property->dependencyPortsDb
		*	a. in PMGRrDbCreate.c write db header into file.
		*	b. in this file (PMGRrAddDependencies) entries are appended to db.
		*	c. upon return to PMGRrDbCreate.c opene as a data base
		*
		*/

		fprintf( dependencyDbStream, "%s%c%s%c%s%c%s%c%c", installedPortDir, 0, installedPortNamePtr, 0, portDependencyDir, 0,
			portDependencyName, 0, LINEFEED );
		/*

		* removed in 0.3.0
		*		MGdbAdd( property->dependencyPortsDb, portName, portDependencyName, portDependencyDir, NULL );
		*/
	}
	if( fclose( dependencyDbStream ) )
	{
		fprintf( stderr, "%s %s error: can not close  %s\n", id, PACKAGE_VERSION, property->dependencyPortsDbFileName );
		perror( "system message" );
		return( 1 );
	}
	free(buffer);
	if( fclose( contentsFileStream ) )
	{
		fprintf( stderr, "%s %s error: can not close  %s\n", id, PACKAGE_VERSION, contentsFileName );
		perror( "system message" );
		return( 1 );
	}
	free(contentsFileName);
	return(0);
}

int	rGetPortDir(char* installedPortNamePtr, char* installedPortDir )
{
	FILE*		contentsFileStream;
	char		id[]			= "rGetPortDir"; 
	char*		address			= NULL;
	char*		buffer			= NULL;
	char*		contentsFileName	= NULL;
	char*		portDirTemp		= NULL;
	int		bufferSize		= 0;

	/*
	 * we need to prepend an '/'
	 */
	installedPortDir[0]	= '/';	
	installedPortDir[1]	= 0;	/* so strcat will start here */
	contentsFileName	= (char*)malloc( 255 );

	strcpy(contentsFileName, PKGDBDIR );
	strcat(contentsFileName, "/");
	strcat(contentsFileName, installedPortNamePtr );
	strcat(contentsFileName, "/");
	strcat(contentsFileName, "+CONTENTS");

	/****************************************/
	/* get +CONTENTS file into buffer	*/
	/****************************************/
	if( ( contentsFileStream = fopen( contentsFileName, "r") ) == 0 )
	{
		fprintf( stderr, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, contentsFileName );
		perror( "system message" );
		return( 1 );
	}
	bufferSize	= MGrFileSize( contentsFileName );
	free( contentsFileName );
	buffer		= (char*)calloc( bufferSize + 1, 1 );
	fread( buffer, 1, bufferSize, contentsFileStream );

	if( ( address = strnstr( buffer, "@comment ORIGIN:", bufferSize ) )  == 0 )
	{
		fprintf( stderr, "%s %s error: \"@comment ORIGIN:\" not found in %s\n", id, PACKAGE_VERSION, contentsFileName );
		fprintf( stderr, "\t\t%s installation is corrupt!\n", installedPortNamePtr );
		fprintf( stderr, "\t\trecomend running \"pkg_delete -f %s\" then manually reinstalling this port\n", installedPortNamePtr );
		return( 1 );
	}

	portDirTemp	= (char*)(address + strlen("@comment ORIGIN:") );
	if( ( address = strnstr( portDirTemp, "\n", 256 ) ) == 0 )
	{
		fprintf( stderr, "%s %s error: \"end of line\" not found in %s\n", id, PACKAGE_VERSION, contentsFileName );
		fprintf( stderr, "\t\t%s installation is corrupt!\n", installedPortNamePtr );
		fprintf( stderr, "\t\trecomend running \"pkg_delete -f %s\" then manually reinstalling this port\n", installedPortNamePtr );
		return( 1 );
	}
	portDirTemp[address-portDirTemp]	= 0;
	strcat( installedPortDir, portDirTemp );

	free( buffer );

	if( fclose( contentsFileStream ) )
	{
		fprintf( stderr, "%s %s error: can not close  %s\n", id, PACKAGE_VERSION, contentsFileName );
		perror( "system message" );
		return( 1 );
	}
	
	return( 0 );
}

int	rVerifyContentsFile( char* installedPortNamePtr )
{
	FILE*		contentsFileStream;
	char		id[]			= "rVerifyContentsFile"; 
	char*		address;
	char*		buffer;
	char*		contentsFileName;
	int		bufferSize		= 0;

	contentsFileName	= (char*)malloc( 255 );
	strcpy(contentsFileName, PKGDBDIR);
	strcat(contentsFileName, "/");
	strcat(contentsFileName, installedPortNamePtr);
	strcat(contentsFileName, "/");
	strcat(contentsFileName, "+CONTENTS");

	if( ( contentsFileStream = fopen(contentsFileName, "r") )  == NULL )
	{
		fprintf( stderr, "%s %s error: could not open %s\n", id, PACKAGE_VERSION, contentsFileName );
		perror( "system message" );
		return( 1 );
	}

	bufferSize	= MGrFileSize( contentsFileName );

	buffer		= (char*)calloc( bufferSize + 1, 1 );

	fread(buffer, 1, bufferSize, contentsFileStream );

	if( fclose( contentsFileStream ) )
	{
		fprintf( stderr, "%s %s error: can not close  %s\n", id, PACKAGE_VERSION, contentsFileName );
		perror( "system message" );
		return( 1 );
	}

	/*********************************/
	/* Verify @comment ORIGIN exists */
	/*********************************/
	address = strnstr( buffer, "@comment ORIGIN:", bufferSize );
	if(!address)
	{
		fprintf( stderr, "%s %s error: \"@comment ORIGIN:\" not found in %s\n", id, PACKAGE_VERSION, contentsFileName );
		fprintf( stderr, "\t\t%s installation is corrupt!\n", installedPortNamePtr );
		fprintf( stderr, "\t\trecomend running \"pkg_delete -f %s\" then manually reinstalling this port\n", installedPortNamePtr );
		return( 1 );
	}

	/*********************************/
	/* Verify @name exists */
	/*********************************/
	if( ( address = strnstr( buffer, "@name ", bufferSize ) ) == 0 )
	{
		fprintf( stderr, "%s %s error: \"@name\" not found in %s\n", id, PACKAGE_VERSION, contentsFileName );
		fprintf( stderr, "\t\t%s installation is corrupt!\n", installedPortNamePtr );
		fprintf( stderr, "\t\trecomend running \"pkg_delete -f %s\" then manually reinstalling this port\n", installedPortNamePtr );
		return( 1 );
	}

	free( contentsFileName );

	free( buffer );

	return( 0 );
}
