/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>
#include	<sgtty.h>

int	nuke( char* leafPortDir );
int	rebuildDb( structProperty* property, int* installedPortsDbQTY );
int	removePort( structProperty* property, char* leafPortName, int installedPortsDbIDX );

int	MGPMrShowLeavesDelete( structProperty* property )
{
	char	exact[]			= "exact";
	char	id[]			= "MGPMrShowLeavesDelete";
	char*	command			= NULL;
	char*	leafPortDir		= NULL;
	char*	leafPortName		= NULL;
	char*	pkgDescFile		= NULL;
	char*	portDirPtr		= NULL;
	char*	portTypePtr		= NULL;
	int	bufferSize		= 0xffff;
	int	installedPortsDbIDX	= 0;
	int	installedPortsDbQTY	= 0;
	int	key			= 1;
	
	command		= calloc( bufferSize, 1 );
	leafPortDir	= calloc( bufferSize, 1 );
	leafPortName	= calloc( bufferSize, 1 );
	pkgDescFile	= calloc( bufferSize, 1 );

	if( ( property->ignoreDb = MGdbOpen( property->ignoreDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->ignoreDbFileName );
		perror( "system" );
		return( 1 );
	}

	if( ( property->installedPortsDb = MGdbOpen( property->installedPortsDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->installedPortsDbFileName );
		perror( "system" );
		return( 1 );
	}

	if( ( property->dependencyPortsDb = MGdbOpen( property->dependencyPortsDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->dependencyPortsDbFileName );
		perror( "system" );
		return( 1 );
	}

	if( ( property->availableDependenciesDb = MGdbOpen( property->availableDependenciesDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->availableDependenciesDbFileName );
		perror( "system" );
		return( 1 );
	}

	installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
	installedPortsDbIDX	= 1;
	while( installedPortsDbIDX < installedPortsDbQTY )
	{
		strcpy( leafPortDir, MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortDir ) );
		strcpy( leafPortName, MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortName ) );		

		/*
		 * check ignored db, don't look at what has been passed over before
		 */
		MGdbGoTop( property->ignoreDb );
		if( MGdbSeek( property->ignoreDb,
				property->fieldignoreDbPortDir,
				leafPortDir,
				property->fieldignoreDbPortDir,
				exact ) )
		{
			installedPortsDbIDX++;
			continue;
		}

		/*
		 * check for no run depends
		 */
		MGdbGoTop( property->dependencyPortsDb );
		portDirPtr	= MGdbSeek( property->dependencyPortsDb,
					property->fieldDependencyDbDependencyPortDir,
					leafPortDir,
					property->fieldDependencyDbDependencyPortDir,
					exact );

		/*
		 * check for no build depends
		 */
		if( !portDirPtr )
		{
			MGdbGoTop( property->availableDependenciesDb );
			portTypePtr	= MGdbSeek( property->availableDependenciesDb,
						property->fieldAvailableDependenciesDbDependencyDir,
						leafPortDir,
						property->fieldAvailableDependenciesDbDependType,
						exact );

			while( portTypePtr && strcmp( portTypePtr, "BUILD_DEPENDS" ) )
			{ 
				portTypePtr	= MGdbSeek( property->availableDependenciesDb,
							property->fieldAvailableDependenciesDbDependencyDir,
							leafPortDir,
							property->fieldAvailableDependenciesDbDependType,
							exact );
			}
		}
		
		if(  !portTypePtr && !portDirPtr )
		{
			if( key != KEY_ENTER )
			{
				system( "clear" );
				fprintf( stdout,"%s\n", SINGLE_LINES );
				fprintf( stdout, "*** for safety run portmanager -u after using this feature for deletion ***\n" );
				fprintf( stdout,"%s\n", SINGLE_LINES );
				fprintf( stdout, "Ports with no ports depending on them AKA \"leaf ports.\"\n" );
				fprintf( stdout, "These ports may be deinstalled because no other ports rely on them to run;\"\n" );
				if( property->buildDependsAreLeaves )
				{
					fprintf( stdout, "however, other ports may rely on these to build.\"\n" );
				}
				fprintf( stdout, "%s\n", SINGLE_LINES );

				fprintf( stdout, "%s dir -=> %s\n\n", leafPortName, leafPortDir );
				while( fflush( stdout ) );

				strcpy( pkgDescFile, PORTSDIR );
				strcat( pkgDescFile, leafPortDir );
				strcat( pkgDescFile, "/pkg-descr" );

				if( MGrIfFileExist( pkgDescFile ) )
				{
					strcpy( command, "cat " );
					strcat( command, pkgDescFile );
					system( command );
					fprintf( stdout, "\n" );
				}
				else
				{
					strcpy( command, "grep COMMENT " );
					strcat( command, PORTSDIR );
					strcat( command, leafPortDir );
					strcat( command, "/Makefile" );
					system( command );
				}
				fprintf( stdout,"%s\n", SINGLE_LINES );
				fprintf( stdout, "\n<SPACE><ENTER> to skip\n<X><ENTER> removes port\n<N><ENTER> *Nuke it*, removes distribution files\n\n" ); 
				fprintf( stdout,"%s\n", SINGLE_LINES );
			}
			key		= getc( stdin );
			switch( key )
			{
				case	KEY_ENTER:
				{
					continue;
				}
				case	KEY_SPACE:
				{
					MGdbAdd( property->ignoreDb, leafPortDir, "-slid <SPACE> skipped", NULL );
					fprintf( stdout, "<SPACE> Skipping %s\n\n", leafPortName );
					break;
				}
				case	KEY_n:
				{
					/* nuke() */
					fprintf( stdout, "<n> Nuking %s\n\n", leafPortName );
					removePort( property, leafPortName, installedPortsDbIDX );
					nuke( leafPortDir );
					rebuildDb( property, &installedPortsDbQTY );
					break;
				}
				case	KEY_N:
				{
					/* nuke() */
					fprintf( stdout, "<N> Nuking %s\n\n", leafPortName );
					removePort( property, leafPortName, installedPortsDbIDX );
					nuke( leafPortDir );
					rebuildDb( property, &installedPortsDbQTY );
					installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
					installedPortsDbIDX	= 1;
					break;
				}
				case	KEY_x:
				{
					/* delete() */
					fprintf( stdout, "<x> Removing %s\n\n", leafPortName );
					removePort( property, leafPortName, installedPortsDbIDX );
					rebuildDb( property, &installedPortsDbQTY );
					installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
					installedPortsDbIDX	= 1;
					break;
				}
				case	KEY_X:
				{
					/* delete() */
					fprintf( stdout, "<X> Removing %s\n\n", leafPortName );
					removePort( property, leafPortName, installedPortsDbIDX );
					rebuildDb( property, &installedPortsDbQTY );
					installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
					installedPortsDbIDX	= 1;
					break;
				}
				default:
				{
					MGdbAdd( property->ignoreDb, leafPortDir, "-slid <DEFAULT> skipped", NULL );
					fprintf( stdout, "Skipping %s\n\n", leafPortName );
					break;
				}
			}
			fprintf( stdout, "\n" );
		}
		installedPortsDbIDX++;
	}
	MGdbDestroy( property->availableDependenciesDb );
	MGdbDestroy( property->dependencyPortsDb );
	MGdbDestroy( property->ignoreDb );
	MGdbDestroy( property->installedPortsDb );
	free( command );
	free( leafPortDir );
	free( leafPortName );
	free( pkgDescFile );
	return(0);
}

/***********************************************************************/
int	removePort( structProperty* property, char* leafPortNamePtr, int installedPortsDbIDX )
{
	char	exact[]		= "exact";
	char*	command		= NULL;

	command		= (char*)calloc( 0xffff, 1 );

	MGdbDelete( property->installedPortsDb, installedPortsDbIDX );

	MGdbGoTop( property->dependencyPortsDb );
	while( MGdbSeek( property->dependencyPortsDb,
		property->fieldDependencyDbPortName,
		leafPortNamePtr,
		property->fieldDependencyDbPortName,
		exact ) )
	{
		MGdbDelete( property->dependencyPortsDb, MGdbGetRecno( property->dependencyPortsDb ) - 1 );
		MGdbGoTop( property->dependencyPortsDb );
	}

	MGdbGoTop( property->availableDependenciesDb );
	while( MGdbSeek( property->availableDependenciesDb,
		property->fieldAvailableDependenciesDbPortName,
		leafPortNamePtr,
		property->fieldAvailableDependenciesDbPortName,
		exact ) )
	{
		MGdbDelete( property->availableDependenciesDb, MGdbGetRecno( property->availableDependenciesDb ) - 1 );
		MGdbGoTop( property->availableDependenciesDb );
	}

	strcpy( command, "pkg_delete -f " );
	strcat( command, leafPortNamePtr );
	fprintf( stdout, "\nexecuting: %s\n", command );
	while( fflush( stdout ) );
	system( command );
	free(command);
 	return( 0 );
}

/***********************************************************************/
int	nuke( char* leafPortDir )
{
	char*	command		= NULL;

	command		= (char*)calloc( 0xffff, 1 );

	strcpy( command, "cd " );
	strcat( command, PORTSDIR );
	strcat( command, leafPortDir );
	strcat( command, "; make distclean" );
	fprintf( stdout, "\nexecuting: %s\n", command );
	while( fflush( stdout ) );
	system( command );
	free(command);
 	return( 0 );
}

/***********************************************************************/
int	rebuildDb( structProperty* property, int* installedPortsDbQTY )
{
	char	id[]	= "rebuildDb";
	
	fprintf( stdout, "%s\n", DOUBLE_LINES );
	fprintf( stdout,"Recreating data bases\n" );
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	MGdbDestroy( property->dependencyPortsDb );
	MGdbDestroy( property->installedPortsDb );
	if( MGPMrCreateInstalledDb( property ) )
	{
		fprintf( stdout, "%s %s error: MGPMrCreateInstalledDb returned an error\n", id, PACKAGE_VERSION );
		return( 1 );
	}
	if( ( property->installedPortsDb = MGdbOpen( property->installedPortsDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->installedPortsDbFileName );
		perror( "system" );
		return( 1 );
	}

	if( ( property->dependencyPortsDb = MGdbOpen( property->dependencyPortsDbFileName ) ) == NULL )
	{
		fprintf( stdout, "%s %s error: failed to open %s\n", id, PACKAGE_VERSION, property->dependencyPortsDbFileName );
		perror( "system" );
		return( 1 );
	}
	*installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
 	return( 0 );
}
