/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/

#include	<libMGPM.h>

int	MGPMrStatus( structProperty* property )
{
	char	exact[]					= "exact";
	char	id[]					= "MGPMrCreateStatusDb"; 
	char*	available				= NULL;
	char*	command					= NULL;
	char*	dependencyDbDependencyPortDirPtr	= NULL;
	char*	dependencyDbDependencyPortNamePtr	= NULL;
	char*	dependencyDbPortDirPtr			= NULL;
	char*	dependencyDbPortNamePtr			= NULL;
	char*	installedPortDir			= NULL;
	char*	installedPortName			= NULL;
	char*	installedPortsDbPortNamePtr		= NULL;
	int	answer					= 0;		
	int	dependencyPortsDbIDX			= 0;
	int	dependencyPortsDbQTX			= 0;
	int	errorCode				= 0;
	int	installedPortsDbIDX			= 1;
	int	installedPortsDbQTY			= 0;
	int	stringSize				= 0;

	available	= (char*)calloc( 255, 1 );
	command		= (char*)calloc( 255, 1 );

	property->dependencyPortsDb	= MGdbOpen( property->dependencyPortsDbFileName );
	property->installedPortsDb	= MGdbOpen( property->installedPortsDbFileName );
	property->oldPortsDb		= MGdbOpen( property->oldPortsDbFileName );
	
	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout,"%s %s info: looking for old installed ports\n", id, PACKAGE_VERSION );
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	/* compare installedPortsDb records to "make describe" of each installed port  */
	/* remove ports that no longer have a Makefile (deleted by CVSand in /usr/ports/MOVED hopefully ) */
	/* add ports whose installedPortsDb names do not match their "make describe" name to oldPortsDb */

	installedPortsDbIDX	= 1;	/* start at one to skip over field names */
	installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
	while( installedPortsDbIDX < installedPortsDbQTY )
	{
		installedPortDir	= MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortDir );

		/* MGPMrMakeDescribe returns the PACKAGE_VERSION of the available port from "cd /usr/ports/; make describe" */
		/*
			change ver 0.2.3 in order to catch changes where the port mantainer forgot to bump revision
			numbers  MGPMrMakeDescribe now sets a change flag which means that even though the port #
			is the same, the make file is either a different size or created at a different date than
			it was when the port was installed
		*/
		MGPMrMakeDescribe( property, installedPortDir, available ); 
		/* remove ports that have been deleted from or moved in the ports collection */		
		/* added 0.1.7 20040804 */
		if( strlen( available ) < 2 )
		{
			installedPortName	= MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortName);
			fprintf( stdout,"%s\n", SINGLE_LINES );
			fprintf( stdout, "%s is no longer in the ports collection, see /usr/ports/MOVED\nis it OK to remove %s from your system? [y/n]\n",
				installedPortDir, installedPortName );
			while( fflush( stdout ) );
			answer	= getc(stdin);

			if( answer != 89 && answer != 121 )
			{
				fprintf( stdout, "answer	= %d\n", answer );
				fprintf( stdout, "portmanager shutting down\n" );
				exit(0);
			}
			answer	= getc(stdin);

			fprintf( stdout, "removing: %s\n", installedPortName );
			stringSize	= strlen("pkg_delete -f ") + strlen(installedPortName) + 1;
			strcpy( command, "pkg_delete -f " );
			strcat( command, installedPortName );
			fprintf( stdout,"%s\n", command );
			fprintf( stdout,"%s\n", SINGLE_LINES );
			while( fflush( stdout ) );
			system( command );
			/*
			 * since installedPortName is no longer in ports collection need to delete record and restart
			 */
			if( ( errorCode = MGdbDelete( property->installedPortsDb, installedPortsDbIDX ) ) )
			{
				fprintf( stdout, "%s %s error: %s returned errorCode %d\n", id, PACKAGE_VERSION, "MGdbDelete", errorCode );
				return( 1 );
			}

			installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
			installedPortsDbIDX	= 1;
			while( fflush( stdout ) );
			continue;
		}

		/* compare installed port name with name from "make describe" */
		/* if installed != available then add portname to oldPortsDb */
		installedPortName	= MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortName );
		installedPortDir	= MGdbGet( property->installedPortsDb, installedPortsDbIDX, property->fieldInstalledPortsDbPortDir);
		if( strcmp( installedPortName,	available ) )
		{
			fprintf( stdout,"have:%-25s status: OLD available:%-25s %-25s\n", installedPortName, available, installedPortDir );
			while( fflush( stdout ) );
			MGdbAdd( property->oldPortsDb, installedPortDir, installedPortName, "OLD", available, installedPortDir, NULL );
			installedPortsDbIDX++;
			continue;
		}		
		fprintf( stdout,"have:%-25s status: CURRENT: %-25s\n", installedPortName, installedPortDir );
		while( fflush( stdout ) );
		installedPortsDbIDX++;
	}

	fprintf( stdout,"%s\n", SINGLE_LINES );
	fprintf( stdout,"%s %s info: looking for installed ports built with old dependencies\n", id, PACKAGE_VERSION );
	fprintf( stdout,"%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	/****************************************************************************/
	/* add any DependencyDir == installedDir && dependencyName != installedName */
	/* to outOfDatePortsDb                                                      */
	/****************************************************************************/
	dependencyPortsDbQTX	= MGdbGetRecordQty( property->dependencyPortsDb );
	dependencyPortsDbIDX	= 1;
	while( dependencyPortsDbIDX < dependencyPortsDbQTX )
	{
		dependencyDbDependencyPortDirPtr	= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX,
								property->fieldDependencyDbDependencyPortDir );
		dependencyDbDependencyPortNamePtr	= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX,
								property->fieldDependencyDbDependencyPortName );
		dependencyDbPortDirPtr			= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX,
								property->fieldDependencyDbPortDir );
		dependencyDbPortNamePtr			= MGdbGet( property->dependencyPortsDb, dependencyPortsDbIDX,
								property->fieldDependencyDbPortName );
		
		MGdbGoTop( property->installedPortsDb );
		installedPortsDbPortNamePtr		= MGdbSeek( property->installedPortsDb, property->fieldInstalledPortsDbPortDir,
								dependencyDbDependencyPortDirPtr, property->fieldInstalledPortsDbPortName, exact );
		
		if( installedPortsDbPortNamePtr && ( strcmp( installedPortsDbPortNamePtr, dependencyDbDependencyPortNamePtr ) == 0 ) )
		{
			dependencyPortsDbIDX++;
			continue;
		}
/*
		MGdbGoTop( property->oldPortsDb );
		if( MGdbSeek( property->oldPortsDb, property->fieldOldPortsDbOldPortDir, dependencyDbPortDirPtr, property->fieldOldPortsDbOldPortDir,
			exact ) == 0 )
		{
*/
			MGdbAdd( property->oldPortsDb, dependencyDbPortDirPtr, dependencyDbPortNamePtr, "built with old ", dependencyDbDependencyPortNamePtr,
				dependencyDbDependencyPortDirPtr, NULL );
				fprintf( stdout,"info:%-25s %s %-25s\n", dependencyDbPortNamePtr, "built with old ", dependencyDbDependencyPortNamePtr );
/*
		}
*/
		dependencyPortsDbIDX++;
	}
	MGdbDestroy( property->dependencyPortsDb );
	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->oldPortsDb );
	free( available );
	free( command );
	return( 0 );
}
