/*
	Copyright (C) 2005  Michael C. Shultz

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:

	1. Redistributions of source code must retain the above copyright
	   notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	Michael C. Shultz
	ringworm01@gmail.com
	Box 3238 Landers, CA 92285
*/
#define		CRASH	100
#define		SKIP	10
#define		OK	0
#include	<libMGPM.h>

typedef struct
{
	char*	afterOptionsFileSize;
	char*	afterOptionsFileTime;
	char*	batchCheck;
	char*	beforeOptionsFileSize;
	char*	beforeOptionsFileTime;
	char*	buffer;
	char*	command;
	char*	commandRestart;
	char*	installedPortName;
	char*	newPortDir;
	char*	newPortName;
	char*	optionsCheck;
	char*	optionsDir;
	char*	startPortCmd;
	char*	startPortDir;
	char*	stopPortCmd;
	char*	stopPortDir;
	char*	workDir;
} structLocalProperty;

int	rCleanDir( char* portDir, char* workDir );
int	rCleanUp( structProperty* property, structLocalProperty* localProperty );
int	rPortBrokeCheck( structProperty* property, char* options, char* oldPortDir, char* oldPortName, char* brokeCheckType );
int	MGPMrUpdate( structProperty* property, char* oldPortDir, char* oldPortName )
{
	FILE*	fileStream					= NULL;
	FILE*	pHandle						= NULL;
	char	BROKEN[]					= "BROKEN";
	char	FORBIDDEN[]					= "FORBIDDEN";
	char	IGNORE[]					= "IGNORE";
	char	START[]						= "/START";
	char	STOP[]						= "/STOP";
	char	backUp[]					= "--back-up";
	char	bu[]						= "-bu";
	char	exact[]						= "exact";
	char	id[]						= "MGPMrUpdate";
	char	ip[]						= "-ip";
	char	l[]						= "-l";
	char	log[]						= "--log";
	char	strike[]					= "0";
	char	y[]						= "-y";
	char	yes[]						= "--yes";
	char*	availableDependenciesDbDependencyDirPtr		= NULL;
	char*	availableDependenciesDbDependencyNamePtr	= NULL;
	char*	bufferBase					= NULL;
	char*	ipPtr						= NULL;
	char*	oldPortDirPtr					= NULL;
	char*	options						= NULL;
	char*	stinker						= NULL;
	char*	startPortDirPtr					= NULL;
	char*	stopPortDirPtr					= NULL;
	char*	strikePtr					= NULL;
	int	availableDependenciesDbIDX			= 0;
	int	availableDependenciesDbQTY			= 0;
	int	availablePortsDbIDX				= 0;
	int	bufferSize					= 0xffff;
	int	errorCode					= 0;
	int	idx						= 0;
	int	installedPortsDbIDX				= 0;
	int	installedPortsDbQTY				= 0;
	int 	kkey						= 1;

	structLocalProperty	localProperty;

	property->optionsChanged	= 0;

	localProperty.afterOptionsFileSize	= (char*)calloc( 511, 1 );
	localProperty.afterOptionsFileTime	= (char*)calloc( 511, 1 );
	localProperty.batchCheck		= (char*)calloc( 511, 1 );
	localProperty.beforeOptionsFileSize	= (char*)calloc( 511, 1 );
	localProperty.beforeOptionsFileTime	= (char*)calloc( 511, 1 );
	localProperty.buffer			= (char*)calloc( bufferSize, 1 );
	localProperty.command			= (char*)calloc( bufferSize, 1 );
	localProperty.commandRestart		= (char*)calloc( bufferSize, 1 );
	localProperty.installedPortName		= (char*)calloc( 511, 1 );
	localProperty.newPortDir		= (char*)calloc( 511, 1 );
	localProperty.newPortName		= (char*)calloc( 511, 1 );
	localProperty.optionsCheck		= (char*)calloc( 511, 1 );
	localProperty.optionsDir		= (char*)calloc( 511, 1 );
	localProperty.startPortCmd		= (char*)calloc( 511, 1 );
	localProperty.startPortDir		= (char*)calloc( 511, 1 );
	localProperty.stopPortCmd		= (char*)calloc( 511, 1 );
	localProperty.stopPortDir		= (char*)calloc( 511, 1 );
	localProperty.workDir			= (char*)calloc( 511, 1 );

	property->availableDependenciesDb	= MGdbOpen( property->availableDependenciesDbFileName );
	property->availablePortsDb		= MGdbOpen( property->availablePortsDbFileName );
	property->commandLineDb			= MGdbOpen( property->commandLineDbFileName );
	property->configDb			= MGdbOpen( property->configDbFileName );
	property->ignoreDb			= MGdbOpen( property->ignoreDbFileName );
	property->installedPortsDb		= MGdbOpen( property->installedPortsDbFileName );
	property->strikesDb			= MGdbOpen( property->strikesDbFileName );

	/* 
	 * store localProperty.workDir
	 */
	strcpy( localProperty.workDir, "cd " );
	strcat( localProperty.workDir, PORTSDIR );
	strcat( localProperty.workDir, oldPortDir );
	strcat( localProperty.workDir, "; make -V WRKDIRPREFIX" );
	localProperty.buffer[0]	= 0;
 	pHandle		= popen( localProperty.workDir, "r" );
	fread( localProperty.buffer, 255, 1, pHandle );
	pclose( pHandle );
	idx	= 0;
	while( idx < 255 )
	{
		if( localProperty.buffer[idx] == LINEFEED || localProperty.buffer[idx] == TAB || localProperty.buffer[idx] == SPACE )
		{
			localProperty.buffer[idx]	= 0;
			break;
		}
		idx++;
	}
	strcpy( localProperty.workDir, localProperty.buffer );

	/* 
	 * store localProperty.optionsDir
	 */
	strcpy( localProperty.optionsDir, "cd " );
	strcat( localProperty.optionsDir, PORTSDIR );
	strcat( localProperty.optionsDir, oldPortDir );
	strcat( localProperty.optionsDir, "; make -V OPTIONSFILE" );
	localProperty.buffer[0]	= 0;
 	pHandle		= popen( localProperty.optionsDir, "r" );
	fread( localProperty.buffer, 255, 1, pHandle );
	pclose( pHandle );
	idx	= 0;
	while( idx < 255 )
	{
		if( localProperty.buffer[idx] == LINEFEED || localProperty.buffer[idx] == TAB || localProperty.buffer[idx] == SPACE )
		{
			localProperty.buffer[idx]	= 0;
			break;
		}
		idx++;
	}
	strcpy( localProperty.optionsDir, localProperty.buffer );

	/*
	 * BEFORE check for /var/db/ports/{port dir name}/options
	 * get size/time of file for later comparison
	 *
	 */
	
	fileStream	= fopen( localProperty.optionsDir, "r" );
	if( fileStream == NULL )	/* no file */
	{
		localProperty.beforeOptionsFileTime[0]	= 0;
		localProperty.beforeOptionsFileSize[0]	= 0;
	}
	else
	{
		fclose( fileStream );
	}
	strcpy( localProperty.beforeOptionsFileTime, MGrIntToString( (int)MGrFileTime( localProperty.optionsDir ) ) );
	strcpy( localProperty.beforeOptionsFileSize, MGrIntToString( MGrFileSize( localProperty.optionsDir ) ) );	

	/*
	 * gather stop/start instructions if they exist
	 */
	MGdbGoTop( property->configDb );
	while( MGdbSeek( property->configDb,
			property->fieldConfigDbKey,
			STOP,
			property->fieldConfigDbKey,
			exact )	
		&&
		strncmp( oldPortDir, 
			MGdbGet( property->configDb,
				MGdbGetRecno( property->configDb ) - 1,
				property->fieldConfigDbValue ),
			strlen( oldPortDir ) ) ==  0 )
	{
		stopPortDirPtr	= MGdbGet( property->configDb,  	
					MGdbGetRecno( property->configDb ) - 1,
					property->fieldConfigDbValue );
		if( stopPortDirPtr[strlen( oldPortDir )] == SPACE )
		{
			strncpy( localProperty.stopPortDir, stopPortDirPtr, strlen( oldPortDir ) );
			strcpy( localProperty.stopPortCmd, stopPortDirPtr + strlen( oldPortDir ) + 1 );
			break;
		}
		else
		{
			break;
		}
	}					
	MGdbGoTop( property->configDb );
	while( MGdbSeek( property->configDb,
			property->fieldConfigDbKey,
			START,
			property->fieldConfigDbKey,
			exact )	
		&&
		strncmp( oldPortDir, 
			MGdbGet( property->configDb,
				MGdbGetRecno( property->configDb ) - 1,
				property->fieldConfigDbValue ),
			strlen( oldPortDir ) ) ==  0 )
	{
		startPortDirPtr	= MGdbGet( property->configDb,  	
					MGdbGetRecno( property->configDb ) - 1,
					property->fieldConfigDbValue );
		if( startPortDirPtr[strlen( oldPortDir )] == SPACE )
		{
			strncpy( localProperty.startPortDir, startPortDirPtr, strlen( oldPortDir ) );
			strcpy( localProperty.startPortCmd, startPortDirPtr + strlen( oldPortDir ) + 1 );
		}
		else
		{
			break;
		}
	}

	/*
	 *	Do update in interactive mode (command line switch -ui )
	 */
	if( property->interactive )
	{
		fprintf( stdout, "ok to update/rebuild %s %s (yes/no/auto yes to all) [y/n/a] [y]?\n",  oldPortDir, oldPortName );
		while( fflush( stdout ) );
		kkey	= getc(stdin);

		switch(kkey)
		{
			case KEY_ENTER:
			{
				break;
			}
			case KEY_A:
			{
				kkey	= getc(stdin);
				property->interactive	= 0;
				break;
			}
			case KEY_a:
			{
				property->interactive	= 0;
				kkey	= getc(stdin);
				break;
			}
			case KEY_Y:
			{
				kkey	= getc(stdin);
				break;
			}
			case KEY_y:
			{
				kkey	= getc(stdin);
				break;
			}
			case KEY_N:
			{
				fprintf( stdout, "\nAdding  %s %s to ignore list\n", oldPortName, oldPortDir );
				MGdbAdd( property->ignoreDb, oldPortDir, "inteactivly skipped", NULL );
				kkey	= getc(stdin);
				rCleanUp( property, &localProperty );
				return(0);
			}
			case KEY_n:
			{
				fprintf( stdout, "\nAdding  %s %s to ignore list\n", oldPortName, oldPortDir );
				MGdbAdd( property->ignoreDb, oldPortDir, "inteactivly skipped", NULL );
				kkey	= getc(stdin);
				rCleanUp( property, &localProperty );
				return(0);
			}
			default:
			{
				fprintf( stdout, "Invalid option. Please enter either Y to upgrade or N to skip\n");
				while( fflush( stdout ) );
				kkey	= getc(stdin);
			}
		}
	}

	/************************************************************************/
	/*			Command "1" " make -V OPTIONS"		*/
	/************************************************************************/

	strcpy( localProperty.batchCheck, "cd " );
	strcat( localProperty.batchCheck, PORTSDIR );
	strcat( localProperty.batchCheck, oldPortDir );
	strcat( localProperty.batchCheck, "; make -V BATCH" );
	
 	pHandle		= popen( localProperty.batchCheck, "r" );
	localProperty.buffer[0]	= 0;
	fread( localProperty.buffer, bufferSize, 1, pHandle );
	pclose( pHandle );
	idx	= 0;
	while( idx < bufferSize )
	{
		if( localProperty.buffer[idx]	== LINEFEED )
		{
			localProperty.buffer[idx]	= 0;
			break;
		}
		idx++;
	}
	if( strlen( localProperty.buffer ) > 0 )	/* BATCH is defined */
	{
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "%s %s:\n\t commands 1 & 2 of 14 OPTIONS check and \"make config\" skipped because BATCH is defined\n",
			id,  PACKAGE_VERSION );
		fprintf( stdout, "\t recommend not defining BATCH so options may at least be set to their defaults\n" );
		while( fflush( stdout ) );
	}
	else
	{
		/*
		 * If no /var/db/ports/{port dir name}/options file exists
		 * yet make -V OPTIONS shows OPTIONS as being defined
		 * make is going to bluescreen, we need to handle that here
		 */

		strcpy( localProperty.optionsCheck, "cd " );
		strcat( localProperty.optionsCheck, PORTSDIR );
		strcat( localProperty.optionsCheck, oldPortDir );
		strcat( localProperty.optionsCheck, "; make -V OPTIONS" );

		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "%s %s command: #1 of 14  %s\n", id, PACKAGE_VERSION, localProperty.optionsCheck );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );

 		pHandle		= popen( localProperty.optionsCheck, "r" );
		localProperty.buffer[0]	= 0;
		fread( localProperty.buffer, bufferSize, 1, pHandle );
		pclose( pHandle );
		/************************************************************************/
		/*			Command "2" " make conf"				*/
		/************************************************************************/
		if( strlen( localProperty.buffer ) > 4 )
		{
			/*
			 * If here we know OPTIONS is defined, now need to know if 
			 * /var/db/ports/{package name}/options exists
			 */
			fileStream	= fopen( localProperty.optionsDir, "r" );
			if( fileStream == NULL )	/* no file */
			{
				if( options )
				{
					strcpy( localProperty.command, "cd " );
					strcat( localProperty.command, PORTSDIR );
					strcat( localProperty.command, oldPortDir );
					strcat( localProperty.command, "; make config " );
					strcat( localProperty.command, options );
				}
				else
				{
					strcpy( localProperty.command, "cd " );
					strcat( localProperty.command, PORTSDIR );
					strcat( localProperty.command, oldPortDir );
					strcat( localProperty.command, "; make config " );
				}	
				fprintf( stdout, "%s\n", SINGLE_LINES );
				fprintf( stdout, "update %s \n", oldPortName );
				fprintf( stdout, "%s %s command: #2 of 14  %s\n", id, PACKAGE_VERSION, localProperty.command );
				fprintf( stdout, "%s\n", SINGLE_LINES );
				while( fflush( stdout ) );
				system(localProperty.command);
			}
			else
			{
				fclose( fileStream );
			}
			strcpy( localProperty.afterOptionsFileTime, MGrIntToString( (int)MGrFileTime( localProperty.optionsDir ) ) );
			strcpy( localProperty.afterOptionsFileSize, MGrIntToString( MGrFileSize( localProperty.optionsDir ) ) );	

			/*
			 * test to see if any options changed
			 */
			if( strlen( localProperty.beforeOptionsFileTime ) !=  strlen( localProperty.afterOptionsFileTime ) 	||
				strlen( localProperty.beforeOptionsFileSize ) !=  strlen( localProperty.afterOptionsFileSize )	||
				strcmp( localProperty.beforeOptionsFileTime, localProperty.afterOptionsFileTime ) 		||
				strcmp( localProperty.beforeOptionsFileSize, localProperty.afterOptionsFileSize ) )
			{
				/*
				 * options changed so need to purge oldPortDir's records
				 * from availablePortsDb and availableDependenciesDb
				 */
				if( property->log )
				{
					strcpy( localProperty.command, "echo \"    options changed so returning " );
					strcat( localProperty.command, oldPortName );
					strcat( localProperty.command, " " );
					strcat( localProperty.command, oldPortDir );
					strcat( localProperty.command, " to out of date pool " );
					strcat( localProperty.command, " \">> /var/log/portmanager.log" );
					system( localProperty.command );
				}
				property->optionsChanged	= 1;
				MGdbGoTop( property->availablePortsDb );
				while( MGdbSeek( property->availablePortsDb,
						property->fieldAvailablePortsDbPortDir,
						oldPortDir,
						property->fieldAvailablePortsDbPortDir,
						exact ) )
				{
					fprintf( stdout, "%s info: purging availablePortsDb of old %s record\n",
						id,
						MGdbGet( property->availablePortsDb,
							MGdbGetRecno( property->availablePortsDb ) - 1,
							property->fieldAvailablePortsDbPortName ) );
					while( fflush( stdout ) );
					property->availablePortsDb	= MGdbDelete( property->availablePortsDb,
											MGdbGetRecno( property->availablePortsDb ) - 1 );
					MGdbGoTop( property->availablePortsDb );
				}

				MGdbGoTop( property->availableDependenciesDb );
				while( MGdbSeek( property->availableDependenciesDb,
						property->fieldAvailableDependenciesDbPortDir,
						oldPortDir,
						property->fieldAvailableDependenciesDbPortDir,
						exact ) )
				{
				fprintf( stdout, "%s info: purging availableDependenciesDb of old %s dependency records %s\n",
					id,
					MGdbGet( property->availableDependenciesDb,
						MGdbGetRecno( property->availableDependenciesDb ) - 1,
						property->fieldAvailableDependenciesDbPortName ),
					MGdbGet( property->availableDependenciesDb,
						MGdbGetRecno( property->availableDependenciesDb ) - 1,
						property->fieldAvailableDependenciesDbDependencyName ) );
					while( fflush( stdout ) );
					MGdbDelete( property->availableDependenciesDb, MGdbGetRecno( property->availableDependenciesDb ) - 1 );
					MGdbGoTop( property->availableDependenciesDb );
				}
				/*
				 * add oldPortDir's new records back into availablePortsDb
				 * and availableDependenciesDb
				 */
				fprintf( stdout, "%s info: adding new/modified %s records\n", id, oldPortDir );
				MGPMrSinglePortCreateStatusDb( property, oldPortDir );
				rCleanUp( property, &localProperty );
				return( 0 );
			}
		}

	}

	/************************************************************************/
	/* 3 strikes check							*/
	/************************************************************************/
	MGdbGoTop( property->strikesDb );
	strikePtr		= MGdbSeek( property->strikesDb, property->fieldStrikesDbPortDir, oldPortDir, property->fieldStrikesIDX, exact );  
	if( strikePtr )
	{
		strcpy( strike, strikePtr );
		if( strike[0]	== '2' )
		{
			MGdbAdd( property->ignoreDb, oldPortDir, "looping, 3rd attempt at make", NULL );
			if( property->log )
			{
				strcpy( localProperty.command, "echo \"make Strike 3 " );
				strcat( localProperty.command, oldPortName );
				strcat( localProperty.command, " " );
				strcat( localProperty.command, oldPortDir );
				strcat( localProperty.command, " marking port as ignore/looping " );
				strcat( localProperty.command, " \">> /var/log/portmanager.log" );
				system( localProperty.command );
			}
			rCleanUp( property, &localProperty );
			return( 0 );
		}
		if( strike[0]	== '1' )
		{
			fprintf( stdout, "%s %s WARNING: This is the second time %s is being built,\n", id, PACKAGE_VERSION, oldPortName );
			fprintf( stdout, "one more attempt and it will be marked as ignore.\n\n" );
			while( fflush( stdout ) );
			MGdbGoTop( property->strikesDb );
			strikePtr		= MGdbSeek( property->strikesDb,
							property->fieldStrikesDbPortDir,
							oldPortDir,
							property->fieldStrikesIDX,
							exact );
			property->strikesDb	= MGdbDelete( property->strikesDb, MGdbGetRecno( property->strikesDb ) - 1 );
			MGdbAdd( property->strikesDb, oldPortDir, "2", NULL );
			if( property->log )
			{
				strcpy( localProperty.command, "echo \"make Strike 2 " );
				strcat( localProperty.command, oldPortName );
				strcat( localProperty.command, " " );
				strcat( localProperty.command, oldPortDir );
				strcat( localProperty.command, " \">> /var/log/portmanager.log" );
				system( localProperty.command );
			}
/*
 * dualOriginCheck( property, oldPortName ); used to be here in old portmanager ver < 0.3.1, seems to be no longer
 * needed, keep this note just in case
 */
		}
	}
	else
	{
		MGdbAdd( property->strikesDb, oldPortDir, "1", NULL );
	}

	MGdbGoTop( property->configDb );
	options		= MGdbSeek( property->configDb, property->fieldConfigDbKey, oldPortDir, property->fieldConfigDbValue, exact );

	/************************************************************************/
	/*			Command "3" " make check-conflicts "		*/
	/************************************************************************/
	if( options )
	{
		strcpy( localProperty.command, "cd " );
		strcat( localProperty.command, PORTSDIR );
		strcat( localProperty.command, oldPortDir );
		strcat( localProperty.command, "; make check-conflicts " );
		strcat( localProperty.command, options );
	}
	else
	{
		strcpy( localProperty.command, "cd " );
		strcat( localProperty.command, PORTSDIR );
		strcat( localProperty.command, oldPortDir );
		strcat( localProperty.command, "; make check-conflicts " );
	}	
	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stdout, "checking for conflicts before building %s\n", oldPortName );
	fprintf( stdout, "%s %s command: #3 of 14  %s\n", id, PACKAGE_VERSION, localProperty.command );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	if( ( errorCode = system(localProperty.command) ) )
	{
		/*
		 * We have a conflicting port installed, pkg_delete -f'ing the stinker
		 *
		 */
		localProperty.buffer[0]	= 0;
		pHandle		= popen( localProperty.command, "r" );
		fread( localProperty.buffer, bufferSize, 1, pHandle );
		if( feof( pHandle ) && !ferror( pHandle ) )
		{
			bufferBase	= strstr( localProperty.buffer, "conflicts with installed package" );
			if( bufferBase )
			{
				/*
				 *
				 * Sample of what to expect in the buffer
				 *
				 * ===>  linux_base-rh-7.3 conflicts with installed package(s):
				 * linux-XFree86-libs-4.3.99.902_2
				 * linux_base-rh-9
				 *
				 */
				bufferBase	= localProperty.buffer
						+ strlen(oldPortName)
						+ strlen( "===>  conflicts with installed package" );

				/*
				 *
				 * Now for the fun part, rather than trying to parse every possible
				 * port in this buffer, we'll just cycle through the install ports database
				 * and see if any of the names show up in this buffer somewhere, they show up
				 * then they are conflicting ports
				 *
				 */
				installedPortsDbIDX	= 0;
				installedPortsDbQTY	= MGdbGetRecordQty( property->installedPortsDb );
				while( installedPortsDbIDX < installedPortsDbQTY )
				{
					strcpy( localProperty.installedPortName, MGdbGet( property->installedPortsDb,
										installedPortsDbIDX,
										property->fieldInstalledPortsDbPortName ) );
					stinker = strstr( bufferBase, localProperty.installedPortName );
					if( stinker )
					{
						fprintf( stdout, "OK to remove conflicting port %s Yy/Nn/Aa [Y]\? ", localProperty.installedPortName );
						while( fflush( stdout ) );
						if( property->autoConflicts == 0 )
						{
							kkey	= getc(stdin);

							switch(kkey)
							{
								case KEY_ENTER:
								{
									break;
								}
								case KEY_A:
								{
									kkey	= getc(stdin);
									property->autoConflicts	= 1;
									break;
								}
								case KEY_a:
								{
									property->autoConflicts	= 1;
									kkey	= getc(stdin);
									break;
								}
								case KEY_Y:
								{
									kkey	= getc(stdin);
									break;
								}
								case KEY_y:
								{
									kkey	= getc(stdin);
									break;
								}
								case KEY_N:
								{
									fprintf( stdout, "\nAdding  %s %s to ignore list\n", oldPortName, oldPortDir );
									while( fflush( stdout ) );
									MGdbAdd( property->ignoreDb, oldPortDir, "conflicts with another port", NULL );
									kkey	= getc(stdin);
									rCleanUp( property, &localProperty );
									return(0);
								}
								case KEY_n:
								{
									fprintf( stdout, "\nAdding  %s %s to ignore list\n", oldPortName, oldPortDir );
									while( fflush( stdout ) );
									MGdbAdd( property->ignoreDb, oldPortDir, "conflicts with another port", NULL );
									kkey	= getc(stdin);
									rCleanUp( property, &localProperty );
									return(0);
								}
								default:
								{
									fprintf( stdout, "Invalid option. Please enter either Y to upgrade or N to skip\n");
									while( fflush( stdout ) );
									kkey	= getc(stdin);
								}
							}
						}
						strcpy( localProperty.command, "pkg_delete -f " );
						strcat( localProperty.command, localProperty.installedPortName );		
						fprintf( stdout, "executing: %s\n", localProperty.command );
						while( fflush( stdout ) );
						errorCode	= system( localProperty.command );
						if( property->log )
						{
							strcpy( localProperty.command, "echo \"removed conflicting port " );
							strcat( localProperty.command, oldPortName );
							strcat( localProperty.command, " " );
							strcat( localProperty.command, oldPortDir );
							strcat( localProperty.command, " \">> /var/log/portmanager.log" );
							system( localProperty.command );
						}
					}
					installedPortsDbIDX++;
				}
			}
			else
			{
				/*
				 * the make check-conflicts command worked but there is no
				 * "conflicts with installed package" in the buffer
				 * let this pass to make and see what happens....
				 */
				errorCode	= 0;
				fprintf( stderr, "no \"conflicts with installed package\" in buffer" );
				while( fflush( stderr ) );
			}
		} 
		else
		{
			/*
			 * the make check-conflicts command failed for some reason so well just
			 * let this pass to make and see what happens....
			 */
			errorCode	= 0;
			fprintf( stderr, "check-conflicts command failed\n" );
			fprintf( stderr, " feof( pHandle )-=>%d ferror( pHandle )-=>%d\n", feof( pHandle ), ferror( pHandle ) );
			while( fflush( stderr ) );
		}
		pclose( pHandle );
	}
	if(errorCode)
	{

		fprintf( stderr, "%s %s error: %s conflicts with another installed port\n", id, PACKAGE_VERSION, oldPortName );
		fprintf( stderr, "you will have to remove the conflicting port before upgrading %s\n", oldPortName );
		fprintf( stderr, "marking %s as ignored\n", oldPortName );
		fprintf( stdout, "%s\n", DOUBLE_LINES );
		while( fflush( stderr ) );
		if( property->log )
		{
			strcpy( localProperty.command, "echo \"      " );
			strcat( localProperty.command, oldPortName );
			strcat( localProperty.command, " " );
			strcat( localProperty.command, oldPortDir );
			strcat( localProperty.command, " conflicts with another installed port so marked ignore\" >> /var/log/portmanager.log" );
			system( localProperty.command );
		}
		MGdbAdd( property->ignoreDb, oldPortDir, "conflicts with another installed port", NULL );
		rCleanUp( property, &localProperty );
		return( 0 );
	}
	/************************************************************************/
	/*			Command "4" " make -V FORBIDDEN "		*/
	/************************************************************************/
	if( ( errorCode = rPortBrokeCheck( property, options, oldPortDir, oldPortName, FORBIDDEN  ) ) )
	{
		switch( errorCode )
		{
			case	CRASH:
			{
				rCleanUp( property, &localProperty );
				return( CRASH );
			}
			case	SKIP:
			{
				rCleanUp( property, &localProperty );
				return( OK );
			}
		}
			
	}

	/************************************************************************/
	/*			Command "5" " make -V BROKEN "		*/
	/************************************************************************/
	if( ( errorCode = rPortBrokeCheck( property, options, oldPortDir, oldPortName, BROKEN  ) ) )
	{
		switch( errorCode )
		{
			case	CRASH:
			{
				rCleanUp( property, &localProperty );
				return( CRASH );
			}
			case	SKIP:
			{
				rCleanUp( property, &localProperty );
				return( OK );
			}
		}
			
	}

	/************************************************************************/
	/*			Command "6" " make -V IGNORE "		*/
	/************************************************************************/
	if( ( errorCode = rPortBrokeCheck( property, options, oldPortDir, oldPortName, IGNORE  ) ) )
	{
		switch( errorCode )
		{
			case	CRASH:
			{
				rCleanUp( property, &localProperty );
				return( CRASH );
			}
			case	SKIP:
			{
				rCleanUp( property, &localProperty );
				return( OK );
			}
		}
			
	}

	/************************************************************************/
	/*			Command "7" " make clean "			*/
	/************************************************************************/
	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stdout, "intitial clean of work directories \n" );
	fprintf( stdout, "%s %s command: #7 of 14:  \n", id, PACKAGE_VERSION );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	availableDependenciesDbQTY	= MGdbGetRecordQty( property->availableDependenciesDb );
	MGdbGoTop( property->availableDependenciesDb );
	oldPortDirPtr	= MGdbSeek( property->availableDependenciesDb,
				property->fieldAvailableDependenciesDbPortDir,
				oldPortDir,
				property->fieldAvailableDependenciesDbPortDir,
				exact );

	availableDependenciesDbIDX	= MGdbGetRecno( property->availableDependenciesDb )-1;

	availableDependenciesDbDependencyDirPtr		= MGdbGet( property->availableDependenciesDb,
								availableDependenciesDbIDX,
								property->fieldAvailableDependenciesDbDependencyDir );

	availableDependenciesDbDependencyNamePtr	= MGdbGet( property->availableDependenciesDb,
								availableDependenciesDbIDX,
								property->fieldAvailableDependenciesDbDependencyName );

	while( oldPortDirPtr && availableDependenciesDbIDX < availableDependenciesDbQTY && strcmp( oldPortDir, oldPortDirPtr ) == 0 )
	{
		rCleanDir( availableDependenciesDbDependencyDirPtr, localProperty.workDir );

		oldPortDirPtr					= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbPortDir );
		availableDependenciesDbDependencyDirPtr		= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbDependencyDir );
		availableDependenciesDbDependencyNamePtr	= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbDependencyName );
		availableDependenciesDbIDX++;
	}
	rCleanDir( oldPortDir, localProperty.workDir );

	/************************************************************************/
	/*			Command "8" make fetch				*/
	/************************************************************************/
	strcpy( localProperty.command, "cd " );
	strcat( localProperty.command, PORTSDIR );
	strcat( localProperty.command, oldPortDir );
	strcat( localProperty.command, "; make fetch " );
	if( options )
	{
		strcat( localProperty.command, options );
	}
	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stdout, "fetch %s \n", oldPortName );
	fprintf( stdout, "%s %s command: #8 of 14  %s\n", id, PACKAGE_VERSION, localProperty.command );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	if( ( errorCode = system(localProperty.command) ) )
	{
		if( property->log )
		{
			strcpy( localProperty.command, "echo \"      " );
			strcat( localProperty.command, oldPortName );
			strcat( localProperty.command, " " );
			strcat( localProperty.command, oldPortDir );
			strcat( localProperty.command, " failed to fetch\" >> /var/log/portmanager.log" );
			system( localProperty.command );
		}
		fprintf( stdout, "%s %s error: make fetch returned an error, adding %s to ignore.db\n", id, PACKAGE_VERSION, oldPortDir );
		while( fflush( stdout ) );
		MGdbAdd( property->ignoreDb, oldPortDir, "failed during make fetch", NULL );
		rCleanUp( property, &localProperty );
		return(0);
	}

	/************************************************************************/
	/*			Command "9" " make"				*/
	/************************************************************************/
	if( options )
	{
		strcpy( localProperty.command, "cd " );
		strcat( localProperty.command, PORTSDIR );
		strcat( localProperty.command, oldPortDir );
		strcat( localProperty.command, "; make " );
		strcat( localProperty.command, options );
	}
	else
	{
		strcpy( localProperty.command, "cd " );
		strcat( localProperty.command, PORTSDIR );
		strcat( localProperty.command, oldPortDir );
		strcat( localProperty.command, "; make " );
	}	
	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stdout, "update %s \n", oldPortName );
	fprintf( stdout, "%s %s command: #9 of 14  %s\n", id, PACKAGE_VERSION, localProperty.command );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	if( ( errorCode = system(localProperty.command) ) )
	{
		if( property->log )
		{
			strcpy( localProperty.command, "echo \"      " );
			strcat( localProperty.command, oldPortName );
			strcat( localProperty.command, " " );
			strcat( localProperty.command, oldPortDir );
			strcat( localProperty.command, " failed during make\" >> /var/log/portmanager.log" );
			system( localProperty.command );
		}
		fprintf( stdout, "%s %s error: make returned an error, adding %s to ignore.db\n", id, PACKAGE_VERSION, oldPortDir );
		while( fflush( stdout ) );
		MGdbAdd( property->ignoreDb, oldPortDir, "failed during make", NULL );
		rCleanUp( property, &localProperty );
		return(0);
	}
	/************************************************************************/
	/*			Command "10" pkg_create -b			*/
	/************************************************************************/
	if( strcmp( oldPortName, "not installed" ) == 0 )
	{
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "backing up installed %s before removing it \n", oldPortName );
		fprintf( stdout, "%s %s command: #10 of 14 N/A - new port\n", id, PACKAGE_VERSION );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
	}
	else
	{
		strcpy( localProperty.command, "cd " );
		strcat( localProperty.command, TEMPDIR );
		strcat( localProperty.command, "; pkg_create -b " );
		strcat( localProperty.command, oldPortName );

		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "backing up installed %s before removing it \n", oldPortName );
		fprintf( stdout, "%s %s command: #10 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		system(localProperty.command);
	}
	/************************************************************************/
	/*			Command "11" " make deinstall "			*/
	/************************************************************************/
	/*
	 * inforced mass update mode portmanager  CANNOT loose its databases
	 */
	if( ( strcmp( oldPortDir, "/sysutils/portmanager" ) == 0 || strcmp( oldPortDir, "/local/sysutils/portmanager" ) == 0 ) &&
		property->pmMode == SINGLE )
	{
		strcpy( localProperty.command, "cd " );
		strcat( localProperty.command, PORTSDIR );
		strcat( localProperty.command, "/sysutils/portmanager; " );
		strcat( localProperty.command, "make -V PKG_DBDIR" );
		pHandle	= popen( localProperty.command, "r" );
		localProperty.buffer[0]	= 0;
		fread( localProperty.buffer, 255, 1, pHandle );
		pclose( pHandle );
		idx	= 0;
		while( idx < 255 )
		{
			if( localProperty.buffer[idx] == LINEFEED || localProperty.buffer[idx] == TAB || localProperty.buffer[idx] == SPACE )
			{
				localProperty.buffer[idx]	= 0;
				break;
			}
			idx++;
		}
		strcpy( localProperty.command, "rm -r " );
		strcat( localProperty.command, localProperty.buffer );
		strcat( localProperty.command, "/portmanager*" );
		fprintf( stdout, "executing: %s\n", localProperty.command );
		while( fflush( stdout ) );
		system( localProperty.command );
	}
	else
	{
		/*
		 * have to do this here before db's are wacked
		 */
		if( ( strcmp( oldPortDir, "/sysutils/portmanager" ) == 0 || strcmp( oldPortDir, "/local/sysutils/portmanager" ) == 0 ) &&
			property->pmMode == MULTI )
		{
			/*
			 * have to do this here before db's are wacked
			 */
			strcpy( localProperty.commandRestart, "portmanager -u" );
			MGdbGoTop( property->commandLineDb );
			while( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, ip,  property->fieldCommandLineDbValue, exact ) ) )
			{
				strcat( localProperty.commandRestart, " " );
				strcat( localProperty.commandRestart, " -ip " );
				strcat( localProperty.commandRestart, ipPtr );
			}
		}
		if( strcmp( oldPortName, "not installed" ) == 0 )
		{
			fprintf( stdout, "%s\n", SINGLE_LINES );
			fprintf( stdout, "removing old %s \n", oldPortName );
			fprintf( stdout, "%s %s command: #11 of 14 N/A new port\n", id, PACKAGE_VERSION );
			fprintf( stdout, "%s\n", SINGLE_LINES );
			while( fflush( stdout ) );
		}
		else
		{
			/*
			 * use localProperty.stopPortCmd if it is defined
			 */
			if( strcmp( oldPortDir, localProperty.stopPortDir ) == 0 )
			{
				fprintf( stdout, "%s %s info: running %s stop command %s\n",
					id,
					PACKAGE_VERSION,
					localProperty.stopPortDir,
					localProperty.stopPortCmd );
				while( fflush( stdout ) );

				localProperty.buffer[0]	= 0;
 				pHandle	= popen( localProperty.stopPortCmd, "r" );
				fread( localProperty.buffer, bufferSize, 1, pHandle );
				pclose( pHandle );
/*
				idx	= 0;
				while( idx < bufferSize )
				{
					if( localProperty.buffer[idx] == LINEFEED 
						||
						localProperty.buffer[idx] == TAB 
						|| 
						localProperty.buffer[idx] == SPACE )
					{
						localProperty.buffer[idx]	= 0;
						break;
					}
					idx++;
				}
*/
				if( property->log )
				{
					strcpy( localProperty.command, "echo \"    stopping " );
					strcat( localProperty.command, oldPortName );
					strcat( localProperty.command, " " );
					strcat( localProperty.command, oldPortDir );
					strcat( localProperty.command, " execution by running " );
					strcat( localProperty.command, localProperty.stopPortCmd );
					strcat( localProperty.command, " \">> /var/log/portmanager.log" );
					system( localProperty.command );
				}
				fprintf( stdout, "%s reply: %s\n", localProperty.stopPortCmd, localProperty.buffer );
				while( fflush( stdout ) );
			}

			/*
			 * remove old port
			 */
			strcpy( localProperty.command, "pkg_delete -f ");
			strcat( localProperty.command, oldPortName);
			fprintf( stdout, "%s\n", SINGLE_LINES );
			fprintf( stdout, "removing old %s \n", oldPortName );
			fprintf( stdout, "%s %s command: #11 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
			fprintf( stdout, "%s\n", SINGLE_LINES );
			while( fflush( stdout ) );
			if( ( system(localProperty.command) ) )
			{
				localProperty.command[0]	= 0;
				strcpy( localProperty.command, "cd " );
				strcat( localProperty.command, PORTSDIR );
				strcat( localProperty.command, oldPortDir );
				strcat( localProperty.command, "; make deinstall " );

				fprintf( stdout, "%s\n", SINGLE_LINES );
				fprintf( stdout, "trying one more way to remove %s...\n", oldPortName );
				fprintf( stdout, "%s %s command: #11 of 14  %s\n", id, PACKAGE_VERSION, localProperty.command );
				fprintf( stdout, "%s\n", SINGLE_LINES );
				while( fflush( stdout ) );
				system(localProperty.command);
			}
		}
	}
	/************************************************************************/
	/*			Command "12" make reinstall			*/
	/************************************************************************/
	if( options )
	{
		strcpy( localProperty.command, "cd " );
		strcat( localProperty.command, PORTSDIR );
		strcat( localProperty.command, oldPortDir );
		strcat( localProperty.command, "; make reinstall " );
		strcat( localProperty.command, options );
	}
	else
	{
		strcpy( localProperty.command, "cd " );
		strcat( localProperty.command, PORTSDIR );
		strcat( localProperty.command, oldPortDir );
		strcat( localProperty.command, "; make reinstall " );
	}	
	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stdout, "installing updated %s \n", oldPortDir );
	fprintf( stdout, "%s %s command: #12 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	if( ( errorCode = system( localProperty.command ) ) )
	{
		if( property->log )
		{
			strcpy( localProperty.command, "echo \"      " );
			strcat( localProperty.command, oldPortName );
			strcat( localProperty.command, " " );
			strcat( localProperty.command, oldPortDir );
			strcat( localProperty.command, " restoring original port from backup\" >> /var/log/portmanager.log" );
			system( localProperty.command );
		}


		strcpy(localProperty.command, "pkg_add ");
		strcat(localProperty.command, TEMPDIR);
		strcat(localProperty.command, "/");
		strcat(localProperty.command, oldPortName);
		strcat(localProperty.command, ".tgz");
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stderr, "restoring original port from backup \n" );
		fprintf( stderr, "%s %s command: #12 of 14!! ***Emergancy restore***  %s\n", id, PACKAGE_VERSION, localProperty.command );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		if( ( errorCode = system( localProperty.command ) ) )
		{
			if( property->log )
			{
				strcpy( localProperty.command, "echo \"      " );
				strcat( localProperty.command, oldPortName );
				strcat( localProperty.command, " " );
				strcat( localProperty.command, oldPortDir );
				strcat( localProperty.command, " failed to restore from backup\" >> /var/log/portmanager.log" );
				system( localProperty.command );
			}
			fprintf( stdout, "%s %s error: FAILED TO RESTORE BACKUP COPY, really sorry ! \n", id, PACKAGE_VERSION );
			fprintf( stdout, "%s %s info: adding %s to ignore.db\n", id, PACKAGE_VERSION, oldPortDir );
			while( fflush( stdout ) );
			MGdbAdd( property->ignoreDb, oldPortDir, "failed during (6) emergancy restore", NULL );
			rCleanUp( property, &localProperty );
			return(0);
		}
		if( property->log )
		{
			strcpy( localProperty.command, "echo \"      " );
			strcat( localProperty.command, oldPortName );
			strcat( localProperty.command, " " );
			strcat( localProperty.command, oldPortDir );
			strcat( localProperty.command, " restored from backup and marked ignore\" >> /var/log/portmanager.log" );
			system( localProperty.command );
		}
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stdout, "%s %s error: RESTORED BACKUP COPY\n",	id, PACKAGE_VERSION );
		fprintf( stdout, "%s %s info: adding %s to ignore.db\n", id, PACKAGE_VERSION, oldPortDir );
		MGdbAdd( property->ignoreDb, oldPortDir, "performed (6) emergancy restore", NULL );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		rCleanUp( property, &localProperty );
		return(0);
	}
	else
	{
		if( strcmp( oldPortName, "not installed" ) )
		{
			strcpy( localProperty.command, "cd " );
			strcat( localProperty.command, TEMPDIR);
			strcat( localProperty.command, "; rm -f ");
			strcat( localProperty.command, oldPortName);
			strcat( localProperty.command, ".tgz" );
			fprintf( stdout, "%s\n", SINGLE_LINES );
			fprintf( stderr, "deleting backup copy, installation of updated %s successful\n", oldPortDir );
			fprintf( stdout, "%s %s command: #12 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
			fprintf( stdout, "%s\n", SINGLE_LINES );
			while( fflush( stdout ) );
			system( localProperty.command );
		}
	}

	/*
	 * use localProperty.startPortCmd if it is defined
	 */
	if( strcmp( oldPortDir, localProperty.startPortDir ) == 0 )
	{
		fprintf( stdout, "%s %s info: running %s start command %s\n",
			id,
			PACKAGE_VERSION,
			localProperty.startPortDir,
			localProperty.startPortCmd );
		while( fflush( stdout ) );
		localProperty.buffer[0]	= 0;
 		pHandle	= popen( localProperty.startPortCmd, "r" );
		fread( localProperty.buffer, bufferSize, 1, pHandle );
		pclose( pHandle );
/*
		idx	= 0;
		while( idx < bufferSize )
		{
			if( localProperty.buffer[idx] == LINEFEED 
				||
				localProperty.buffer[idx] == TAB 
				|| 
				localProperty.buffer[idx] == SPACE )
			{
				localProperty.buffer[idx]	= 0;
				break;
			}
			idx++;
		}
*/
		if( property->log )
		{
			strcpy( localProperty.command, "echo \"    starting " );
			strcat( localProperty.command, oldPortName );
			strcat( localProperty.command, " " );
			strcat( localProperty.command, oldPortDir );
			strcat( localProperty.command, " execution by running " );
			strcat( localProperty.command, localProperty.startPortCmd );
			strcat( localProperty.command, " \">> /var/log/portmanager.log" );
			system( localProperty.command );
		}

		fprintf( stdout, "%s reply: %s\n", localProperty.startPortCmd, localProperty.buffer );
		while( fflush( stdout ) );
	}

	/************************************************************************/
	/*			Command "13" make package			*/
	/************************************************************************/
	if( property->backUp )
	{
		if( options )
		{
			strcpy( localProperty.command, "cd " );
			strcat( localProperty.command, PORTSDIR );
			strcat( localProperty.command, oldPortDir );
			strcat( localProperty.command, "; make package " );
			strcat( localProperty.command, options );
		}
		else
		{
			strcpy( localProperty.command, "cd " );
			strcat( localProperty.command, PORTSDIR );
			strcat( localProperty.command, oldPortDir );
			strcat( localProperty.command, "; make package " );
		}	
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stderr, "making package of updated %s \n", oldPortDir );
		fprintf( stdout, "%s %s command: #13 of 14 %s\n", id, PACKAGE_VERSION, localProperty.command );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
		system(localProperty.command);
	}
	else
	{
		fprintf( stdout, "%s\n", SINGLE_LINES );
		fprintf( stderr, "--back-up not selected skipping making package of updated %s \n", oldPortDir );
		fprintf( stdout, "%s\n", SINGLE_LINES );
		while( fflush( stdout ) );
	}
	/************************************************************************/
	/*			Command "14" make clean				*/
	/************************************************************************/
	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stderr, "terminal clean of work directories\n" );
	fprintf( stdout, "%s %s command: #14 of 14\n", id, PACKAGE_VERSION  );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );

	/*
	 * if portmanager updated there won't be any databases, so need to clean the old fashoned way
	 */
	if( ( strcmp( oldPortDir, "/sysutils/portmanager" ) == 0 || strcmp( oldPortDir, "/local/sysutils/portmanager" ) == 0 ) &&
		property->pmMode == MULTI )
	{
		rCleanDir( oldPortDir, localProperty.workDir );
	}
	else
	{
		availableDependenciesDbQTY	= MGdbGetRecordQty( property->availableDependenciesDb );
		MGdbGoTop( property->availableDependenciesDb );
		oldPortDirPtr	= MGdbSeek( property->availableDependenciesDb,
					property->fieldAvailableDependenciesDbPortDir,
					oldPortDir,
					property->fieldAvailableDependenciesDbPortDir,
					exact );

		availableDependenciesDbIDX	= MGdbGetRecno( property->availableDependenciesDb )-1;

		availableDependenciesDbDependencyDirPtr		= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbDependencyDir );

		availableDependenciesDbDependencyNamePtr	= MGdbGet( property->availableDependenciesDb,
									availableDependenciesDbIDX,
									property->fieldAvailableDependenciesDbDependencyName );

		while( oldPortDirPtr && availableDependenciesDbIDX < availableDependenciesDbQTY && strcmp( oldPortDir, oldPortDirPtr ) == 0 )
		{
			rCleanDir( availableDependenciesDbDependencyDirPtr, localProperty.workDir );

			oldPortDirPtr					= MGdbGet( property->availableDependenciesDb,
										availableDependenciesDbIDX,
										property->fieldAvailableDependenciesDbPortDir );
			availableDependenciesDbDependencyDirPtr		= MGdbGet( property->availableDependenciesDb,
										availableDependenciesDbIDX,
										property->fieldAvailableDependenciesDbDependencyDir );
			availableDependenciesDbDependencyNamePtr	= MGdbGet( property->availableDependenciesDb,
										availableDependenciesDbIDX,
										property->fieldAvailableDependenciesDbDependencyName );
			availableDependenciesDbIDX++;
		}
		rCleanDir( oldPortDir, localProperty.workDir );
	}

	/*
	 * portmanager auto restart after updating itself
	 */
	if( ( strcmp( oldPortDir, "/sysutils/portmanager" ) == 0 || strcmp( oldPortDir, "/local/sysutils/portmanager" ) == 0 ) &&
		property->pmMode == MULTI )
	{
		/*
		 * reconstruct the command line
		 */
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, l,  property->fieldCommandLineDbValue, exact ) ) )
		{
			strcat( localProperty.commandRestart, " " );
			strcat( localProperty.commandRestart, "-l" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, log,  property->fieldCommandLineDbValue, exact ) ) )
		{
			strcat( localProperty.commandRestart, " " );
			strcat( localProperty.commandRestart, "-l" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, bu,  property->fieldCommandLineDbValue, exact ) ) )
		{
			strcat( localProperty.commandRestart, " " );
			strcat( localProperty.commandRestart, "-bu" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, backUp,  property->fieldCommandLineDbValue, exact ) ) )
		{
			strcat( localProperty.commandRestart, " " );
			strcat( localProperty.commandRestart, "-bu" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, y,  property->fieldCommandLineDbValue, exact ) ) )
		{
			strcat( localProperty.commandRestart, " " );
			strcat( localProperty.commandRestart, "-y" );
		}
		MGdbGoTop( property->commandLineDb );
		if( ( ipPtr = MGdbSeek( property->commandLineDb, property->fieldCommandLineDbKey, yes,  property->fieldCommandLineDbValue, exact ) ) )
		{
			strcat( localProperty.commandRestart, " " );
			strcat( localProperty.commandRestart, "--yes" );
		}
		strcat( localProperty.commandRestart, " --resume" );
		fprintf(stdout, "%s\n", SINGLE_LINES);
		fprintf(stdout,	"portmanager just updated itself, executing:\n\t%s\n", localProperty.commandRestart );
		fprintf(stdout, "%s\n", SINGLE_LINES);
		while( fflush( stdout ) );
		system( localProperty.commandRestart );
		exit( 0 );

	}

	/*
	 * If forced mode Mark port as being built
	 */
	if( property->pmMode || property->forced )
	{
		MGdbGoTop( property->availablePortsDb );
		if( strncmp( MGdbSeek( property->availablePortsDb,
					property->fieldAvailablePortsDbPortDir,
					oldPortDir,
					property->fieldAvailablePortsDbFlag,
					exact ),
					"2", 1 ) )
		{
			availablePortsDbIDX		= MGdbGetRecno( property->availablePortsDb ) - 1;
			strcpy( localProperty.newPortName, MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbPortName ) );
			strcpy( localProperty.newPortDir, MGdbGet( property->availablePortsDb, availablePortsDbIDX, property->fieldAvailablePortsDbPortDir ) );
			property->availablePortsDb = MGdbDelete( property->availablePortsDb, availablePortsDbIDX );
			MGdbAdd( property->availablePortsDb, localProperty.newPortName, localProperty.newPortDir, "2", NULL );			
		}
	}
	rCleanUp( property, &localProperty );
	return( 0 );
}
/****************************************/
/****************************************/
/****************************************/
int	rCleanUp( structProperty* property, structLocalProperty* localProperty )
{
	MGdbDestroy( property->availableDependenciesDb );
	MGdbDestroy( property->availablePortsDb );
	MGdbDestroy( property->commandLineDb );
	MGdbDestroy( property->configDb );
	MGdbDestroy( property->ignoreDb );
	MGdbDestroy( property->installedPortsDb );
	MGdbDestroy( property->strikesDb );

	free( localProperty->afterOptionsFileSize );
	free( localProperty->afterOptionsFileTime );
	free( localProperty->batchCheck );
	free( localProperty->beforeOptionsFileSize );
	free( localProperty->beforeOptionsFileTime );
	free( localProperty->buffer );
	free( localProperty->command );
	free( localProperty->commandRestart );
	free( localProperty->installedPortName );
	free( localProperty->newPortDir );
	free( localProperty->newPortName );
	free( localProperty->optionsCheck );
	free( localProperty->optionsDir );
	free( localProperty->startPortCmd );
	free( localProperty->startPortDir );
	free( localProperty->stopPortCmd );
	free( localProperty->stopPortDir );
	free( localProperty->workDir );

	return(0);
}
/****************************************/
/****************************************/
/****************************************/
int	rPortBrokeCheck( structProperty* property, char* options, char* oldPortDir, char* oldPortName, char* brokeCheckType )
{
	FILE*	pHandle		= NULL;
	char	id[]		= "rPortBrokeCheck";
	char*	buffer		= NULL;
	char*	command		= NULL;
	char*	comment		= NULL;
	int	bufferSize	= 0xffff;
	
	buffer	= (char*)calloc( bufferSize, 1 );
	command	= (char*)calloc( 255, 1 );
	comment	= (char*)calloc( 255, 1 );

	strcpy( command, "cd " );
	strcat( command, PORTSDIR );
	strcat( command, oldPortDir );
	strcat( command, "; make  -V " );
	strcat( command, brokeCheckType );

	if( options )
	{
		strcat( command, " "  );
		strcat( command, options );
	}

	fprintf( stdout, "%s\n", SINGLE_LINES );
	fprintf( stdout, "checking if port marked %s before building %s %s\n", brokeCheckType, oldPortName, oldPortDir );
	fprintf( stdout, "%s %s command: %s\n", id, PACKAGE_VERSION, command );
	fprintf( stdout, "%s\n", SINGLE_LINES );
	while( fflush( stdout ) );
	if( ferror( pHandle = popen( command, "r" ) ) )
	{
		fprintf( stderr, "%s error: reading make -V %s into tempStream\n", id, brokeCheckType );
		perror( "system message" );
		pclose( pHandle );
		free( comment );
		free( command );
		free( buffer );
		return( CRASH );
	}
	buffer[0]	= 0;
	fread( buffer, bufferSize, 1, pHandle );
	if( ferror( pHandle ) == 0 && strlen( buffer ) > 1 )
	{
		fprintf( stdout, "%s %s error: %s %s marked %s, adding to ignore.db\n", id, PACKAGE_VERSION, oldPortName, oldPortDir, brokeCheckType );
		while( fflush( stdout ) );
		strcpy( comment, "port marked " );
		strcat( comment, brokeCheckType );
		MGdbAdd( property->ignoreDb, oldPortDir, comment, NULL );
		if( property->log )
		{
			strcpy( command, "echo \"      " );
			strcat( command, oldPortName );
			strcat( command, " " );
			strcat( command, oldPortDir );
			strcat( command, " marked " );
			strcat( command, brokeCheckType );
			strcat( command, " port not installed/updated\" >> /var/log/portmanager.log" );
			system( command );
		}
		pclose( pHandle );
		free( buffer );	
		free( command );
		free( comment );
		return( SKIP );
	}
	pclose( pHandle );
	free( buffer );	
	free( command );
	free( comment );
	return( OK );
}
/**********************************/
/**********************************/
/**********************************/
int	rCleanDir( char* portDir, char* workDir )
{
	FILE*	pHandle		= NULL;
	char*	buffer		= NULL;
	char*	command		= NULL;
	char*	curDir		= NULL;
	int	bufferSize	= 0xffff;
	int	idx		= 0;

	buffer		= (char*)calloc( bufferSize, 1 );
	command		= (char*)calloc( bufferSize, 1 );
	curDir		= (char*)calloc( bufferSize, 1 );

	/*
	 * get .CURDIR
	 */
	strcpy( curDir, "cd " );
	strcat( curDir, PORTSDIR );
	strcat( curDir, portDir );
	strcat( curDir, "; make -V .CURDIR" );
	buffer[0]	= 0;
	pHandle		= popen( curDir, "r" );
	fread( buffer, 255, 1, pHandle );
	pclose( pHandle );
	idx	= 0;
	while( idx < 255 )
	{
		if( buffer[idx] == LINEFEED || buffer[idx] == TAB || buffer[idx] == SPACE )
		{
			buffer[idx]	= 0;
			break;
		}
		idx++;
	}
	strcpy( curDir, buffer );
	
	strcpy( command, "rm -r " );
	strcat( command, workDir );
	strcat( command, curDir );
/*
	strcat( command, "/work >/dev/null 2>&1" );
*/
strcat( command, "/work" );
	fprintf( stdout, "===>  Cleaning for %s\n", portDir );	
fprintf( stdout, " %s\n", command );	
	while( fflush( stdout ) );
	system( command );

	free( buffer );	
	free( command );
	free( curDir );
	return( 0 );
}
